@@ -25,20 +25,28 @@ from e3.anod.action import (
 from e3.anod.deps import Dependency
 from e3.anod.error import AnodError
 from e3.anod.package import UnmanagedSourceBuilder
-from e3.anod.spec import has_primitive
+from e3.anod.spec import has_primitive, fetch_attr
 from e3.collection.dag import DAG
+from e3.electrolyt.plan import PlanActionEnv
 from e3.env import BaseEnv
 from e3.error import E3Error
 
 if TYPE_CHECKING:
-    from typing import Callable, Dict, FrozenSet, List, NoReturn, Optional, Tuple, Union
+    from typing import (
+        cast,
+        NoReturn,
+        Optional,
+        Tuple,
+    )
+    from collections.abc import Callable
     from e3.anod.action import Action
     from e3.anod.package import SourceBuilder
-    from e3.anod.spec import Anod
+    from e3.anod.spec import Anod, PRIMITIVE
     from e3.anod.loader import AnodSpecRepository
     from e3.anod.sandbox import SandBox
     from e3.collection.dag import VertexID
     from e3.platform import Platform
+    from e3.mypy import assert_never
 
     # spec name, build env, target env, host env, qualifier, kind, source name
     CacheKeyType = Tuple[
@@ -46,6 +54,7 @@ if TYPE_CHECKING:
     ]
     ResolverType = Callable[[Action, Decision], bool]
 
+
 logger = e3.log.getLogger("anod.context")
 
 
@@ -54,10 +63,10 @@ class SchedulingError(E3Error):
 
     def __init__(
         self,
-        message: Union[str, List[str]],
+        message: str | list[str],
         origin: Optional[str] = None,
         uid: Optional[VertexID] = None,
-        initiators: Optional[List[VertexID]] = None,
+        initiators: Optional[list[VertexID]] = None,
     ):
         """Scheduling error initialization.
 
@@ -117,17 +126,17 @@ class AnodContext:
 
         self.tree = DAG()
         self.root = Root()
-        self.dependencies: Dict[str, Dict[str, Tuple[Dependency, Anod]]] = {}
+        self.dependencies: dict[str, dict[str, tuple[Dependency, Anod]]] = {}
         self.add(self.root)
-        self.cache: Dict[CacheKeyType, Anod] = {}
-        self.sources: Dict[str, Tuple[str, SourceBuilder]] = {}
+        self.cache: dict[CacheKeyType, Anod] = {}
+        self.sources: dict[str, tuple[str, SourceBuilder]] = {}
 
     def load(
         self,
         name: str,
         env: Optional[BaseEnv],
         qualifier: Optional[str],
-        kind: str,
+        kind: PRIMITIVE,
         sandbox: Optional[SandBox] = None,
         source_name: Optional[str] = None,
     ) -> Anod:
@@ -184,7 +193,7 @@ class AnodContext:
         root: Action,
         left: Action,
         right: Action,
-    ) -> None:
+    ) -> Decision:
         """Add a decision node.
 
         This create the following subtree inside the dag::
@@ -200,6 +209,7 @@ class AnodContext:
         decision_action = decision_class(root, left, right)
         self.add(decision_action, left, right)
         self.connect(root, decision_action)
+        return decision_action
 
     def connect(self, action: Action, *args: Action) -> None:
         """Add predecessors to a node.
@@ -225,7 +235,7 @@ class AnodContext:
         """
         return self.tree[key]
 
-    def predecessors(self, action: Action) -> List[Action]:
+    def predecessors(self, action: Action) -> list[Action]:
         """Retrieve predecessors of a given action.
 
         :param action: the parent action
@@ -253,22 +263,64 @@ class AnodContext:
                 )
         self.tree.add_tag(vertex_id, {"plan_line": plan_line, "plan_args": plan_args})
 
+    def add_plan_action(
+        self, plan_action_env: PlanActionEnv, sandbox: Optional[SandBox] = None
+    ) -> Optional[Action]:
+        """Add an Anod action to the context.
+
+        :param plan_action_env: the PlanActionEnv object as returned by PlanContext
+        :param sandbox: the SandBox object that will be used to run commands
+        :return: the root added action or None if this is not an anod action
+        """
+        action_name = plan_action_env.action
+        if not action_name.startswith("anod_") or plan_action_env.module is None:
+            return None
+
+        primitive = action_name.replace("anod_", "", 1)
+        if (
+            primitive != "build"
+            and primitive != "install"
+            and primitive != "test"
+            and primitive != "source"
+        ):
+            logger.warning(f"Unknown primtive {primitive}")
+            return None
+        elif TYPE_CHECKING:
+            primitive = cast(PRIMITIVE, primitive)
+
+        return self.add_anod_action(
+            name=plan_action_env.module,
+            env=self.default_env
+            if plan_action_env.default_build
+            else BaseEnv.from_env(plan_action_env),
+            primitive=primitive,
+            qualifier=plan_action_env.qualifier,
+            source_packages=plan_action_env.source_packages,
+            upload=plan_action_env.push_to_store,
+            plan_line=plan_action_env.plan_line,
+            plan_args=plan_action_env.plan_args,
+            sandbox=sandbox,
+        )
+
     def add_anod_action(
         self,
         name: str,
         env: BaseEnv,
-        primitive: str,
+        primitive: PRIMITIVE,
         qualifier: Optional[str] = None,
-        source_packages: Optional[List[str]] = None,
+        source_packages: Optional[list[str]] = None,
         upload: bool = True,
         plan_line: Optional[str] = None,
         plan_args: Optional[dict] = None,
         sandbox: Optional[SandBox] = None,
     ) -> Action:
-        """Add an Anod action to the context.
+        """Add an Anod action to the context (internal function).
+
+        Note that using add_anod_action should be avoided when possible
+        and replaced by a call to add_plan_action.
 
         :param name: spec name
-        :param env: spec environment
+        :param env: context in which to load the spec
         :param primitive: spec primitive
         :param qualifier: qualifier
         :param source_packages: if not empty only create the specified list of
@@ -278,6 +330,7 @@ class AnodContext:
         :param plan_line: corresponding line:linenumber in the plan
         :param plan_args: action args after plan execution, taking into
             account plan context (such as with defaults(XXX):)
+        :param sandbox: the SandBox object that will be used to run commands
         :return: the root added action
         """
         # First create the subtree for the spec
@@ -310,7 +363,7 @@ class AnodContext:
             if build_action is not None:
                 spec = build_action.data
                 if spec.component is not None and upload:
-                    upload_bin: Union[UploadBinaryComponent, UploadSourceComponent]
+                    upload_bin: UploadBinaryComponent | UploadSourceComponent
                     if spec.has_package:
                         upload_bin = UploadBinaryComponent(spec)
                     else:
@@ -330,26 +383,29 @@ class AnodContext:
             for el in self.predecessors(result):
                 if isinstance(el, BuildOrDownload):
                     el.set_decision(BuildOrDownload.INSTALL, plan_line)
+        elif primitive != "source" and primitive != "test":
+            assert_never()
         return result
 
     def add_spec(
         self,
         name: str,
         env: BaseEnv,
-        primitive: str,
+        primitive: PRIMITIVE,
         qualifier: Optional[str] = None,
-        source_packages: Optional[List[str]] = None,
+        source_packages: Optional[list[str]] = None,
         expand_build: bool = True,
         source_name: Optional[str] = None,
         plan_line: Optional[str] = None,
         plan_args: Optional[dict] = None,
         sandbox: Optional[SandBox] = None,
         upload: bool = False,
-    ) -> Union[Build, CreateSources, CreateSource, Install, Test]:
+        force_download: bool = False,
+    ) -> Build | CreateSources | CreateSource | Install | Test:
         """Expand an anod action into a tree (internal).
 
         :param name: spec name
-        :param env: spec environment
+        :param env: context in which to load the spec
         :param primitive: spec primitive
         :param qualifier: qualifier
         :param source_packages: if not empty only create the specified list of
@@ -365,6 +421,7 @@ class AnodContext:
             the given sandbox
         :param upload: if True consider uploads to the store (sources and
             binaries)
+        :param force_download: if True force a download
         """
 
         def add_action(data: Action, connect_with: Optional[Action] = None) -> None:
@@ -390,11 +447,6 @@ class AnodContext:
             spec_instance.deps[dep.local_name] = dep_instance
 
         # Initialize a spec instance
-        e3.log.debug(
-            "add spec: name:{}, qualifier:{}, primitive:{}".format(
-                name, qualifier, primitive
-            )
-        )
         spec = self.load(
             name,
             qualifier=qualifier,
@@ -403,10 +455,13 @@ class AnodContext:
             sandbox=sandbox,
             source_name=source_name,
         )
-        result: Union[Build, CreateSources, CreateSource, Install, Test]
+        result: Build | CreateSources | CreateSource | Install | Test
 
         # Initialize the resulting action based on the primitive name
         if primitive == "source":
+            if not has_primitive(spec, "source"):
+                raise SchedulingError(f"spec {name} does not support primitive source")
+
             if source_name is not None:
                 result = CreateSource(spec, source_name)
 
@@ -445,13 +500,18 @@ class AnodContext:
                     self.connect(result, sub_result)
 
         elif primitive == "build":
+            if not has_primitive(spec, "build"):
+                raise SchedulingError(
+                    f"spec {name} does not support primitive build for"
+                    f" platform {env.platform} and qualifier '{qualifier}'"
+                )
             result = Build(spec)
         elif primitive == "test":
             result = Test(spec)
         elif primitive == "install":
             result = Install(spec)
-        else:  # defensive code
-            raise ValueError("add_spec error: %s is not known" % primitive)
+        else:
+            assert_never()
 
         # If this action is directly linked with a plan line make sure
         # to register the link between the action and the plan even
@@ -461,17 +521,32 @@ class AnodContext:
                 vertex_id=result.uid, plan_line=plan_line, plan_args=plan_args
             )
 
-        if (
+        if primitive == "install" and force_download:
+            # We have an "download" dependency explicit in the plan
+            # Make sure to record the BuildOrDownload decision
+            if result in self:
+                action_preds = self.predecessors(result)
+                if action_preds:
+                    dec = action_preds[0]
+                    if isinstance(dec, BuildOrDownload):
+                        dec.set_decision(
+                            which=BuildOrDownload.INSTALL, decision_maker=plan_line
+                        )
+
+        elif (
             primitive == "install"
             and not spec.has_package
             and has_primitive(spec, "build")
+            and not force_download
         ):
             if plan_line is not None and plan_args is not None:
                 # We have an explicit call to install() in the plan but the
                 # spec has no binary package to download.
                 raise SchedulingError(
-                    "error in plan at {}: "
-                    "install should be replaced by build".format(plan_line)
+                    f"error in plan at {plan_line}: "
+                    "install should be replaced by build - "
+                    f"the spec {spec.name} has a build primitive "
+                    "but does not define a package"
                 )
             # Case in which we have an install dependency but no install
             # primitive. In that case the real dependency is a build tree
@@ -535,9 +610,13 @@ class AnodContext:
                     sandbox=sandbox,
                     upload=upload,
                 )
-                self.add_decision(
+                decision = self.add_decision(
                     BuildOrDownload, result, build_action, download_action
                 )
+                if force_download:
+                    decision.set_decision(
+                        which=BuildOrDownload.INSTALL, decision_maker=plan_line
+                    )
             else:
                 self.connect(result, download_action)
 
@@ -573,13 +652,12 @@ class AnodContext:
                             co = Checkout(checkout, self.repo.repos[checkout])
                             add_action(co, result)
 
-        # Look for dependencies
-        spec_dependencies = []
-        if (
-            "%s_deps" % primitive in dir(spec)
-            and getattr(spec, "%s_deps" % primitive) is not None
-        ):
-            spec_dependencies += getattr(spec, "%s_deps" % primitive)
+        # Look for dependencies. Consider that "None" means "no dependency".
+        spec_dependencies = list(fetch_attr(spec, f"{primitive}_deps", None) or [])
+
+        source_spec_dependencies_names = {
+            d.name for d in spec_dependencies if d.kind == "source"
+        }
 
         for e in spec_dependencies:
             if isinstance(e, Dependency):
@@ -604,12 +682,13 @@ class AnodContext:
                 child_action = self.add_spec(
                     name=e.name,
                     env=e.env(spec, self.default_env),
-                    primitive=e.kind,
+                    primitive=e.kind if e.kind != "download" else "install",
                     qualifier=e.qualifier,
                     plan_args=None,
                     plan_line=plan_line,
                     sandbox=sandbox,
                     upload=upload,
+                    force_download=e.kind == "download",
                 )
 
                 add_dep(
@@ -621,24 +700,43 @@ class AnodContext:
                     # We have a build tree dependency that produced a
                     # subtree starting with an install node. In that case
                     # we expect the user to choose BUILD as decision.
-                    dec = self.predecessors(child_action)[0]
-                    if isinstance(dec, BuildOrDownload):
-                        dec.add_trigger(
-                            result,
-                            BuildOrDownload.BUILD,
-                            plan_line if plan_line is not None else "unknown line",
-                        )
+                    child_action_preds = self.predecessors(child_action)
+                    if child_action_preds:
+                        dec = child_action_preds[0]
+                        if isinstance(dec, BuildOrDownload):
+                            dec.add_trigger(
+                                result,
+                                BuildOrDownload.BUILD,
+                                plan_line if plan_line is not None else "unknown line",
+                            )
 
                 # Connect child dependency
                 self.connect(result, child_action)
 
         # Look for source dependencies (i.e sources needed)
-        if "%s_source_list" % primitive in dir(spec):
-            source_list = getattr(spec, f"{primitive}_source_list")
+        source_list = fetch_attr(spec, f"{primitive}_source_list", None)
+        if source_list is not None:
             for s in source_list:
                 # set source builder
                 if s.name in self.sources:
-                    s.set_builder(self.sources[s.name])
+                    sb_spec, sb = self.sources[s.name]
+                    if (
+                        sb_spec != spec.name
+                        and sb_spec not in source_spec_dependencies_names
+                        # ignore unmanaged source builders which do not
+                        # create many issues (no need to find the source
+                        # builder to apply patches, update repositories, ...)
+                        # and this creates too many warnings in production that
+                        # we do not have time to fix
+                        and not isinstance(sb, UnmanagedSourceBuilder)
+                    ):
+                        logger.warning(
+                            f"{spec.name}.anod ({primitive}): source {s.name}"
+                            f" coming from {sb_spec} but there is no"
+                            f" source_pkg dependency for {sb_spec} in {primitive}_deps",
+                        )
+                    s.set_builder(sb)
+
                 # set other sources to compute source ignore
                 s.set_other_sources(source_list)
                 # add source install node
@@ -700,7 +798,7 @@ class AnodContext:
 
         :param action: action to consider
         :param decision: decision to resolve
-        :raise SchedulingError
+        :raise: SchedulingError
         """
         if decision.choice is None and decision.expected_choice in (
             Decision.LEFT,
@@ -789,7 +887,7 @@ class AnodContext:
             return cls.decision_error(action, decision)
 
     @classmethod
-    def always_create_source_resolver(cls, action, decision):
+    def always_create_source_resolver(cls, action: Action, decision: Decision) -> bool:
         """Force source creation when scheduling a plan."""
         if isinstance(action, CreateSource):
             return True
@@ -804,8 +902,8 @@ class AnodContext:
         :param resolver: a function that helps the scheduler resolve cases
             for which a decision should be taken
         """
-        rev = self.tree.reverse_graph()
-        uploads: List[Tuple[Upload, FrozenSet[VertexID]]] = []
+        rev = self.tree.reverse_graph(enable_checks=False)
+        uploads: list[tuple[Upload, frozenset[VertexID]]] = []
         dag = DAG()
 
         # Retrieve existing tags
@@ -889,7 +987,7 @@ class AnodContext:
                             ]
                             raise SchedulingError(
                                 e.messages, uid=uid, initiators=initiators
-                            )
+                            ) from e
                 else:
                     # An action is scheduled only if one of its successors is
                     # scheduled.
