@@ -7,8 +7,9 @@ import e3.anod.error
 from e3.env import BaseEnv
 
 if TYPE_CHECKING:
-    from typing import Any, Hashable, Optional
-    from e3.anod.spec import Anod
+    from typing import Any, Hashable, Literal, Optional
+    from e3.anod.spec import Anod, DEPENDENCY_PRIMITIVE
+    from e3.mypy import assert_never
 
 
 class BuildVar:
@@ -30,6 +31,9 @@ class BuildVar:
 
 
 class Dependency:
+
+    kind: DEPENDENCY_PRIMITIVE
+
     def __init__(
         self,
         name: str,
@@ -39,7 +43,10 @@ class Dependency:
         build: Optional[str] = None,
         qualifier: Optional[str] = None,
         local_name: Optional[str] = None,
-        require: str = "build_tree",
+        require: Literal["build_tree"]
+        | Literal["installation"]
+        | Literal["download"]
+        | Literal["source_pkg"] = "build_tree",
         track: bool = False,
         **kwargs: Any,
     ) -> None:
@@ -62,7 +69,7 @@ class Dependency:
             in deps and makedeps dictionaries. It allows importing twice
             the same anod module with different qualifers or platforms
         :param require: can be 'build_tree' (to force a local build),
-            'installation' or 'source_pkg'
+            'installation', 'download', or 'source_pkg'
         :param track: if True, track all source packages metadata and include
             them in the local metadata.
         :param kwargs: other parameters valid in some API that we ignore now
@@ -75,16 +82,26 @@ class Dependency:
         self.build = build
         self.qualifier = qualifier
         self.local_name = local_name if local_name is not None else name
-        if require not in ("build_tree", "installation", "source_pkg"):
+        if require not in (
+            "build_tree",
+            "download",
+            "installation",
+            "source_pkg",
+        ):
             raise e3.anod.error.SpecError(
-                "require should be build_tree, installation or source_pkg"
-                " not %s." % require
+                f"require should be build_tree, download, installation,"
+                f" or source_pkg not {require}."
             )
-        self.kind = {
-            "build_tree": "build",
-            "installation": "install",
-            "source_pkg": "source",
-        }[require]
+        if require == "build_tree":
+            self.kind = "build"
+        elif require == "download":
+            self.kind = "download"
+        elif require == "installation":
+            self.kind = "install"
+        elif require == "source_pkg":
+            self.kind = "source"
+        else:
+            assert_never()
         self.track = track
 
     def env(self, parent: Anod, default_env: BaseEnv) -> BaseEnv:
