@@ -2,10 +2,9 @@ from __future__ import annotations
 
 import inspect
 import os
-import sys
 import types
-
 import yaml
+
 from typing import TYPE_CHECKING
 
 import e3.hash
@@ -17,11 +16,26 @@ from e3.fs import ls
 logger = e3.log.getLogger("anod.loader")
 
 if TYPE_CHECKING:
-    from typing import Callable, Dict, List, Optional
+    from typing import Any, Optional
+    from collections.abc import Callable
     from types import ModuleType
     from e3.anod.spec import Anod
 
 
+class SpecConfig:
+    """Contain specification files configuration.
+
+    :ivar spec_dir: path to the anod specs
+    :ivar repositories: dict containing the list of repositories metadata
+        (content of config/repositories)
+    """
+
+    def __init__(self) -> None:
+        # Both values are set by AnodSpecRepository init
+        self.spec_dir = ""
+        self.repositories: dict[str, Any] = {}
+
+
 class AnodSpecRepository:
     """Anod spec repository.
 
@@ -33,21 +47,31 @@ class AnodSpecRepository:
     files.
     """
 
-    def __init__(self, spec_dir: str, spec_config: Optional[dict] = None):
+    def __init__(
+        self,
+        spec_dir: str,
+        spec_config: Any = None,
+        # Ideally should be spec_config: Optional[SpecConfig] = None,
+        # We keep it to Any to avoid mypy issues on other projects
+        extra_repositories_config: Optional[dict] = None,
+    ):
         """Initialize an AnodSpecRepository.
 
         :param spec_dir: directory containing the anod specs.
         :param spec_config: dictionary containing the configuration for this
             AnodSpecRepository
+        :param extra_repositories_config: first read the configuration from
+            <spec_dir>/config/repositories.yaml and update the result with
+            extra_repositories_config
         """
         logger.debug("initialize spec repository (%s)", spec_dir)
 
         if not os.path.isdir(spec_dir):
-            raise SandBoxError("spec directory %s does not exist" % spec_dir)
+            raise SandBoxError(f"spec directory {spec_dir} does not exist")
         self.spec_dir = spec_dir
         self.api_version = __version__
         self.specs = {}
-        self.repos: Dict[str, Dict[str, str]] = {}
+        self.repos: dict[str, dict[str, str]] = {}
 
         # Look for all spec files and data files
         spec_list = {
@@ -98,6 +122,23 @@ class AnodSpecRepository:
             with open(repo_file) as fd:
                 self.repos = yaml.safe_load(fd)
 
+        if extra_repositories_config:
+            for repo_name, repo_data in extra_repositories_config.items():
+                if repo_name in self.repos:
+                    self.repos[repo_name].update(repo_data)
+                else:
+                    self.repos[repo_name] = repo_data
+
+        # Make sure that all revision are strings and not floats
+        for repo_conf in self.repos.values():
+            if "revision" in repo_conf:
+                repo_conf["revision"] = str(repo_conf["revision"])
+
+        if spec_config is None:
+            spec_config = SpecConfig()
+        spec_config.spec_dir = self.spec_dir
+        spec_config.repositories = self.repos
+
         # Declare spec prolog
         prolog_file = os.path.join(spec_dir, "prolog.py")
         self.prolog_dict = {"spec_config": spec_config, "__spec_repository": self}
@@ -129,12 +170,12 @@ class AnodSpecRepository:
 
         :param name: name of the spec to load
         """
-        assert name in self.specs, "spec %s not found" % name
+        assert name in self.specs, f"spec {name} not found"
         return self.specs[name].load(self)
 
 
 class AnodModule:
-    def __init__(self, name: str, path: str, data: List[str]):
+    def __init__(self, name: str, path: str, data: list[str]):
         """Initialize an AnodModule instance.
 
         :param name: module name
@@ -185,8 +226,8 @@ class AnodModule:
             logger.error("exception: %s", e)
             logger.error("cannot load code of %s", self.name)
             raise SandBoxError(
-                origin="load", message="invalid spec code for %s" % self.name
-            ).with_traceback(sys.exc_info()[2])
+                origin="load", message=f"invalid spec code for {self.name}"
+            ) from e
 
         # At this stage we have loaded completely the module. Now we need to
         # look for a subclass of Anod. Use python inspection features to
@@ -200,6 +241,13 @@ class AnodModule:
                 and value.__module__ == mod_name
                 and "Anod" in (k.__name__ for k in value.__mro__)
             ):
+                # Reject class named "Anod" this is reserved to the base
+                # class and can cause some issues when reused
+                if value.__name__ == "Anod":
+                    raise SandBoxError(
+                        f"{self.name}.anod must not use Anod as a class name", "load"
+                    )
+
                 # This class is a child of Anod so register it.
                 # Note that even if we won't use directly the
                 # module we need to keep a reference on it in order
@@ -216,8 +264,8 @@ class AnodModule:
                 self.anod_class.api_version = repository.api_version  # type: ignore
                 return value
 
-        logger.error("spec %s does not contains an Anod subclass", self.name)
-        raise SandBoxError("cannot find Anod subclass in %s" % self.path, "load")
+        logger.error(f"spec {self.name} does not contains an Anod subclass")
+        raise SandBoxError(f"cannot find Anod subclass in {self.path}", "load")
 
 
 def spec(name: str) -> Callable[..., Anod]:
