@@ -11,7 +11,15 @@ from e3.fs import VCS_IGNORE_LIST, mkdir, rm, sync_tree
 
 
 if TYPE_CHECKING:
-    from typing import Callable, Dict, Final, List, Literal, NoReturn, Optional, Union
+    from typing import (
+        Any,
+        Callable,
+        Dict,
+        Final,
+        Literal,
+        NoReturn,
+        Optional,
+    )
     from e3.anod.spec import Anod
 
     PrepareSrcCB = Callable[[Dict[str, Dict[str, str]], str], None]
@@ -43,6 +51,17 @@ class Package:
         self.publish = publish
         self.version = version
 
+    @property
+    def is_simple_archive(self) -> bool:
+        """
+        Check if package creation can be bypassed.
+
+        If true it means that package creation can be bypassed
+        in some context such as user mode. By default it is
+        True
+        """
+        return True
+
     def pkg_name(self, anod_instance: Anod) -> str:
         """Return the final package filename.
 
@@ -130,18 +149,18 @@ class Source:
         self.unpack_cmd = unpack_cmd
         self.remove_root_dir = remove_root_dir
         self.builder: Optional[SourceBuilder] = None
-        self.other_sources: List[Source] = []
+        self.other_sources: list[Source] = []
 
     def set_builder(self, builder_function: SourceBuilder) -> None:
         """Set the SourceBuilder associated to this Source object."""
         self.builder = builder_function
 
-    def set_other_sources(self, other_sources: List[Source]) -> None:
+    def set_other_sources(self, other_sources: list[Source]) -> None:
         """Get the list of other sources to compute ``ignore`` property."""
         self.other_sources = other_sources
 
     @property
-    def ignore(self) -> List[str]:
+    def ignore(self) -> list[str]:
         """Return list of paths to ignore when installing the source.
 
         By default, a source package is first unpacked and then install
@@ -176,9 +195,9 @@ class SourceBuilder:
         self,
         name: str,
         fullname: Callable[[], str],
-        checkout: Optional[List[str]],
+        checkout: Optional[list[str]],
         prepare_src: Optional[PrepareSrcCB] = None,
-        apply_patch: Optional[Union[Literal[1], ApplyPatchCB]] = None,
+        apply_patch: Optional[Literal[1] | ApplyPatchCB] = None,
         kind: str = "source",
     ):
         """Define a builder for the source package.
@@ -208,14 +227,14 @@ class SourceBuilder:
         self.repositories = {}  # type: ignore
         self.kind = kind
         self.from_spec = None
-        self.source_query: Optional[Dict[str, Optional[str]]] = None
+        self.source_query: Optional[dict[str, Optional[str]]] = None
         self.__fullname = fullname
         self.__prepare_src = prepare_src
         self.__apply_patch = apply_patch
 
-    def fullname(self, *args, **kwargs):
+    def fullname(self, *args: Any, **kwargs: Any) -> str:
         # ??? add support for the GPL mode
-        return self.__fullname(*args, **kwargs)
+        return self.__fullname(*args, **kwargs)  # type: ignore
 
     @property
     def prepare_src(self) -> Optional[PrepareSrcCB]:
@@ -230,7 +249,7 @@ class SourceBuilder:
         # Else provide a default function if we have exactly 1 checkout
         if not self.checkout:
             raise e3.anod.error.SpecError(
-                "no checkout associated to the builder %s" % self.name, "prepare_src"
+                f"no checkout associated to the builder {self.name}", "prepare_src"
             )
         elif len(self.checkout) > 1:
             raise e3.anod.error.SpecError(
@@ -240,7 +259,7 @@ class SourceBuilder:
 
         # Set default function (a basic sync_tree call) that ignore
         # .svn, .git, .cvs, .cvsignore and .gitignore files
-        def default_prepare_src(repos: Dict[str, Dict[str, str]], dest: str) -> None:
+        def default_prepare_src(repos: dict[str, dict[str, str]], dest: str) -> None:
             sync_tree(
                 list(repos.values())[0]["working_dir"], dest, ignore=VCS_IGNORE_LIST
             )
@@ -264,7 +283,7 @@ class SourceBuilder:
             # Unused parameters
             del r, p, d
             raise e3.anod.error.AnodError(
-                "no apply_patch function defined in SourceBuilder %s" % self.name,
+                f"no apply_patch function defined in SourceBuilder {self.name}",
                 "apply_path",
             )
 
