@@ -23,10 +23,18 @@ SUPPORTED_API = (__version__, "1.5")
 # only the version 1.4 and 1.5. Default is still 1.4
 
 logger = e3.log.getLogger("anod")
+spec_logger = e3.log.getLogger("anod.spec")
 
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, IO, List, Optional, Sequence, Union
+    from typing import (
+        Any,
+        IO,
+        Literal,
+        Optional,
+        Union,
+    )
+    from collections.abc import Callable, Sequence
     from e3.anod.buildspace import BuildSpace
     from e3.anod.sandbox import SandBox
     from e3.env import BaseEnv
@@ -35,6 +43,22 @@ if TYPE_CHECKING:
     import e3.anod.package
     import e3.anod.sandbox
 
+    BUILD_PRIMITIVE = Literal["build"]
+    DOWNLOAD_PRIMITIVE = Literal["download"]
+    INSTALL_PRIMITIVE = Literal["install"]
+    TEST_PRIMITIVE = Literal["test"]
+    SOURCE_PRIMITIVE = Literal["source"]
+
+    # Anod Dependency can target a build, install, or source
+    DEPENDENCY_PRIMITIVE = Union[
+        BUILD_PRIMITIVE, DOWNLOAD_PRIMITIVE, INSTALL_PRIMITIVE, SOURCE_PRIMITIVE
+    ]
+
+    # Supported primitivies are build, install, source, and test
+    PRIMITIVE = Union[
+        BUILD_PRIMITIVE, INSTALL_PRIMITIVE, SOURCE_PRIMITIVE, TEST_PRIMITIVE
+    ]
+
 
 def check_api_version(version: str) -> None:
     """Make sure there are no API mismatch.
@@ -49,7 +73,7 @@ def check_api_version(version: str) -> None:
         )
 
 
-def parse_command(command: Sequence[str], build_space: BuildSpace) -> List[str]:
+def parse_command(command: Sequence[str], build_space: BuildSpace) -> list[str]:
     """Parse a command line formatting each string.
 
     :param command: the command line (a list of string)
@@ -60,9 +84,11 @@ def parse_command(command: Sequence[str], build_space: BuildSpace) -> List[str]:
     return [e3.text.format_with_dict(c, cmd_dict) for c in command]
 
 
-def has_primitive(anod_instance: Anod, name: str) -> bool:
+def has_primitive(anod_instance: Anod, name: Literal["download"] | PRIMITIVE) -> bool:
     """Return True if the primitive `name` is supported.
 
+    Note that download is currently considered as a primitive in that context.
+
     :param anod_instance: an Anod instance
     :param name: name of the primitive ('build', 'install'...)
     """
@@ -75,7 +101,34 @@ def has_primitive(anod_instance: Anod, name: str) -> bool:
         is_primitive: bool = func.is_primitive
     except AttributeError:
         return False
-    return is_primitive
+
+    if is_primitive:
+        if func.require is None:
+            return True
+        else:
+            return func.require(anod_instance)
+    else:
+        return False
+
+
+def fetch_attr(instance: Any, name: str, default_value: Any) -> Any:
+    """Return an attribute or the default value if missing.
+
+    Unlike `getattr(instance, name, default_value)`, this works only on
+    attributes present in the class (so class attributes or properties) and
+    this does not hide AttributeError exceptions that getting an existing
+    attribute might raise.
+    """
+    # Singleton used to determine whether call to gettattr returns the default
+    # value (i.e. attribute is missing) or just returns the attribute value (it
+    # could be None).
+    sentinel = object()
+
+    return (
+        default_value
+        if getattr(type(instance), name, sentinel) is sentinel
+        else getattr(instance, name)
+    )
 
 
 class Anod:
@@ -91,8 +144,8 @@ class Anod:
         class MyProduct(Anod):
             pass
 
-    All attributes starting with spec_ are reserved by the driver and must not
-    be overwritten.
+    All attributes starting with ``spec_`` are reserved by the driver and must
+    not be overwritten.
 
     Several attributes are set when loading the spec:
 
@@ -130,7 +183,7 @@ class Anod:
     sandbox: Optional[SandBox] = None
     name = ""
     api_version = ""
-    data_files = ()
+    data_files: tuple[str, ...] = ()
 
     # API
     Dependency = e3.anod.deps.Dependency
@@ -143,7 +196,11 @@ class Anod:
     ThirdPartySourceBuilder = e3.anod.package.ThirdPartySourceBuilder
 
     def __init__(
-        self, qualifier: str, kind: str, jobs: int = 1, env: Optional[BaseEnv] = None
+        self,
+        qualifier: str,
+        kind: PRIMITIVE,
+        jobs: int = 1,
+        env: Optional[BaseEnv] = None,
     ):
         """Initialize an Anod instance.
 
@@ -154,7 +211,7 @@ class Anod:
         :param env: alternate platform environment
         :raise: SpecError
         """
-        self.deps: Dict[str, Anod] = {}
+        self.deps: dict[str, Anod] = {}
 
         self.kind = kind
         self.jobs = jobs
@@ -163,7 +220,7 @@ class Anod:
         self.__build_space: Optional[BuildSpace] = None
 
         # Default spec logger
-        self.log = e3.log.getLogger("anod.spec")
+        self.log = spec_logger
 
         # Set spec environment
         self.env = e3.env.BaseEnv.from_env(env)
@@ -186,13 +243,13 @@ class Anod:
         self.qualifier = qualifier
 
         # Default build space name is the spec name
-        if "build_space_name" not in dir(self):
+        build_space_name = fetch_attr(self, "build_space_name", None)
+        if build_space_name is None:
             self.build_space_name = self.name
 
         # UID of the spec instance
         self.uid = ".".join(
             (
-                self.env.build.machine,
                 self.env.platform,
                 self.build_space_name,
                 self.kind,
@@ -203,7 +260,7 @@ class Anod:
         self._config: Optional[dict] = None
 
         # Hold the result of the pre function
-        self._pre: Optional[Dict[str, Any]] = None
+        self._pre: Optional[dict[str, Any]] = None
 
     @property
     def build_space(self) -> BuildSpace:
@@ -226,13 +283,6 @@ class Anod:
             name=self.build_space_name, platform=self.env.platform
         )
 
-    def bind_to_config(self, config: dict) -> None:
-        """Bind an Anod instance to a config.
-
-        :param config: a dictionary-like object
-        """
-        self._config = config
-
     def load_config_file(
         self,
         extended: bool = False,
@@ -256,7 +306,8 @@ class Anod:
         else:
             filename = "{}{}".format(self.name, "-" + suffix if suffix else "")
         assert filename in self.data_files, "invalid data file: {} ({})".format(
-            filename, ", ".join(self.data_files),
+            filename,
+            ", ".join(self.data_files),
         )
         filename = os.path.join(self.spec_dir, filename + ".yaml")
 
@@ -302,6 +353,7 @@ class Anod:
         pre: Optional[Callable[[Anod], dict]] = None,
         post: Optional[Callable[..., None]] = None,
         version: Optional[Callable[..., str]] = None,
+        require: Optional[Callable[[Anod], bool]] = None,
     ) -> Callable:
         """Declare an anod primitive.
 
@@ -315,11 +367,14 @@ class Anod:
         :param version: None or a callback function returning the version
             that will be evaluated as a string. This callback is called
             after running the primitive
+        :param require: None or a special function to call before running the
+            primitive. The function takes a unique parameter `self` and
+            returns a boolean
         :raise: AnodError
         """
 
-        def primitive_dec(f, pre=pre, post=post, version=version):
-            def primitive_func(self, *args, **kwargs):
+        def primitive_dec(f, pre=pre, post=post, version=version):  # type: ignore
+            def primitive_func(self, *args, **kwargs):  # type: ignore
                 self.log.debug("%s %s starts", self.name, f.__name__)
 
                 result = False
@@ -344,22 +399,16 @@ class Anod:
 
                     # And return the result
                     return result
-                except AnodError:
-                    self.log.exception("%s %s fails", self.name, f.__name__)
-                    raise AnodError(
-                        "%s %s fails (AnodError exception in primitive)"
-                        % (self.name, f.__name__)
-                    )
-                except Exception as e:
-                    self.log.exception("%s %s fails", self.name, f.__name__)
-                    raise AnodError(
-                        f"{self.name} {f.__name__} fails (got exception: {e})"
-                    )
+                except Exception as err:
+                    error_msg = f"{self.name} {f.__name__} fails"
+                    self.log.exception(error_msg)
+                    raise AnodError(error_msg) from err
 
             primitive_func.is_primitive = True
             primitive_func.pre = pre
             primitive_func.post = post
             primitive_func.version = version
+            primitive_func.require = require
             return primitive_func
 
         return primitive_dec
@@ -391,7 +440,7 @@ class Anod:
         return self.uid
 
     @property
-    def source_pkg_build(self) -> Optional[List[e3.anod.package.SourceBuilder]]:
+    def source_pkg_build(self) -> Optional[list[e3.anod.package.SourceBuilder]]:
         """Return list of SourceBuilder defined in the specification file."""
         return None
 
@@ -399,7 +448,7 @@ class Anod:
         self,
         *command: str,
         parse_shebang: bool = True,
-        output: Union[STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = None,
+        output: STDOUT_VALUE | DEVNULL_VALUE | PIPE_VALUE | str | IO | None = None,
         python_executable: None = None,
         **kwargs: Any,
     ) -> e3.os.process.Run:
