@@ -20,16 +20,54 @@ import e3.os.fs
 
 
 if TYPE_CHECKING:
-    from typing import Callable, List, Optional, Sequence
+    from typing import Literal, Optional, Text, Union
+    from collections.abc import Callable, Sequence
+    from os import PathLike
+    from e3.mypy import assert_never
+
+    UnpackAutoRemoveDirType = Literal["auto"]
+    RemoveRootDirType = Union[bool, Literal["auto"]]
+
+    TAR_GZ = Literal["tar.gz"]
+    TAR_BZ2 = Literal["tar.bz2"]
+    TAR_XZ = Literal["tar.xz"]
+    TAR = Literal["tar"]
+    ZIP = Literal["zip"]
+
+else:
+    TAR_GZ = "tar.gz"
+    TAR_BZ = "tar.bz2"
+    TAR_XZ = "tar.xz"
+    TAR = "tar"
+    ZIP = "zip"
 
 logger = e3.log.getLogger("archive")
 
+if sys.platform == "win32":
+    # On Windows force the executable bit on all files. This ensures that when
+    # using cygwin unziped files get an executable bit set (the executable
+    # does not exist in win32 but is simulated in Cygwin).
+
+    class E3ZipInfo(zipfile.ZipInfo):
+        @classmethod
+        def from_file(cls, *args, **kwargs):  # type: ignore
+            result = super().from_file(*args, **kwargs)
+            result.external_attr = (0o555 << 16) | result.external_attr
+            return result
+
+    zipfile.ZipInfo = E3ZipInfo  # type: ignore
+
 
 class E3ZipFile(zipfile.ZipFile):
     """Override default ZipFile with attributes preservation."""
 
-    def _extract_member(self, member, path, pwd):
-        result = super()._extract_member(member, path, pwd)
+    def _extract_member(
+        self,
+        member: Text | zipfile.ZipInfo,
+        path: Optional[str | PathLike[str]],
+        pwd: Optional[bytes],
+    ) -> str:
+        result = super()._extract_member(member, path, pwd)  # type: ignore
 
         if sys.platform != "win32":
             # Try to preserve attributes on non Windows platforms as
@@ -44,7 +82,7 @@ class E3ZipFile(zipfile.ZipFile):
                 attr = member.external_attr >> 16 & 0x1FF
                 if attr != 0:
                     os.chmod(result, attr)
-            except AttributeError:
+            except AttributeError:  # defensive code
                 pass
         return result
 
@@ -60,10 +98,12 @@ def is_known_archive_format(filename: str) -> bool:
     :return: True if the path corresponding to a supported archive format
     """
     ext = e3.fs.extension(filename)
-    return ext in (".tar.gz", ".tgz", ".tar.bz2", ".tar", ".zip")
+    return ext in (".tar.gz", ".tgz", ".tar.bz2", ".tar.xz", ".tar", ".zip")
 
 
-def check_type(filename: str, force_extension: Optional[str] = None) -> str:
+def check_type(
+    filename: str, force_extension: Optional[str] = None
+) -> TAR_GZ | TAR_BZ2 | TAR_XZ | TAR | ZIP:
     """Return the archive extension.
 
     Internal function used by create_archive and unpack_archive.
@@ -80,36 +120,40 @@ def check_type(filename: str, force_extension: Optional[str] = None) -> str:
         or filename.endswith(".tgz")
         or (force_extension is not None and force_extension in [".tar.gz", ".tgz"])
     ):
-        ext = "tar.gz"
+        return "tar.gz"
     elif filename.endswith(".tar.bz2") or (
         force_extension is not None and force_extension == ".tar.bz2"
     ):
-        ext = "tar.bz2"
+        return "tar.bz2"
+    elif filename.endswith(".tar.xz") or (
+        force_extension is not None and force_extension == ".tar.xz"
+    ):
+        return "tar.xz"
     elif filename.endswith(".tar") or (
         force_extension is not None and force_extension == ".tar"
     ):
-        ext = "tar"
+        return "tar"
     elif filename.endswith(".zip") or (
         force_extension is not None and force_extension == ".zip"
     ):
-        ext = "zip"
+        return "zip"
     else:
         raise ArchiveError(
-            origin="unpack_archive", message='unknown format "%s"' % filename
+            origin="unpack_archive", message=f'unknown format "{filename}"'
         )
-    return ext
 
 
 def unpack_archive(
     filename: str,
     dest: str,
     selected_files: Optional[Sequence[str]] = None,
-    remove_root_dir: bool = False,
+    remove_root_dir: RemoveRootDirType = False,
     unpack_cmd: Optional[Callable[..., None]] = None,
     force_extension: Optional[str] = None,
     delete: bool = False,
-    ignore: Optional[List[str]] = None,
+    ignore: Optional[list[str]] = None,
     preserve_timestamps: bool = True,
+    tmp_dir_root: Optional[str] = None,
 ) -> None:
     """Unpack an archive file (.tgz, .tar.gz, .tar or .zip).
 
@@ -136,6 +180,10 @@ def unpack_archive(
     :param preserve_timestamps: if False and remove_root_dir is True, and the
         target directory exists, ensure that updated files get their timestamp
         updated to current time.
+    :param tmp_dir_root: If not None the temporary directory used to extract the
+        archive will be created in tmp_dir_root directory. If None the temporary
+        directory is created in the destination directory. This argument only
+        has an effect when remove_root_dir is True.
 
     :raise ArchiveError: in case of error
 
@@ -145,11 +193,11 @@ def unpack_archive(
     # First do some checks such as archive existence or destination directory
     # existence.
     if not os.path.isfile(filename):
-        raise ArchiveError(origin="unpack_archive", message="cannot find %s" % filename)
+        raise ArchiveError(origin="unpack_archive", message=f"cannot find {filename}")
 
     if not os.path.isdir(dest):
         raise ArchiveError(
-            origin="unpack_archive", message="dest dir %s does not exist" % dest
+            origin="unpack_archive", message=f"dest dir {dest} does not exist"
         )
 
     if selected_files is None:
@@ -171,14 +219,14 @@ def unpack_archive(
     # If remove_root_dir is set then extract to a temp directory first.
     # Otherwise extract directly to the final destination
     if remove_root_dir:
-        tmp_dest = tempfile.mkdtemp(
-            prefix="", dir=os.path.dirname(os.path.abspath(dest))
-        )
+        if tmp_dir_root is None:
+            tmp_dir_root = os.path.dirname(os.path.abspath(dest))
+        tmp_dest = tempfile.mkdtemp(prefix="", dir=tmp_dir_root)
     else:
         tmp_dest = dest
 
     try:
-        if ext in ("tar", "tar.bz2", "tar.gz"):
+        if ext == "tar" or ext == "tar.bz2" or ext == "tar.gz" or ext == "tar.xz":
             try:
                 # Set the right mode
                 mode = "r:"
@@ -186,6 +234,8 @@ def unpack_archive(
                     mode += "bz2"
                 elif ext.endswith("gz"):
                     mode += "gz"
+                elif ext.endswith("xz"):
+                    mode += "xz"
                 # Extract tar files
                 with closing(tarfile.open(filename, mode=mode)) as fd:
                     check_selected = set(selected_files)
@@ -204,7 +254,7 @@ def unpack_archive(
                                 return True
                         return False
 
-                    dirs: List[str] = []
+                    dirs: list[str] = []
 
                     # IMPORTANT: don't use the method extract. Always use the
                     # extractall function. Indeed extractall will set file
@@ -226,7 +276,7 @@ def unpack_archive(
 
                         if check_selected:
                             raise ArchiveError(
-                                "unpack_archive", "Cannot untar %s " % filename
+                                "unpack_archive", f"Cannot untar {filename} "
                             )
 
                         fd.extractall(path=tmp_dest, members=member_list)
@@ -235,10 +285,11 @@ def unpack_archive(
 
             except tarfile.TarError as e:
                 raise ArchiveError(
-                    origin="unpack_archive", message=f"Cannot untar {filename} ({e})",
-                ).with_traceback(sys.exc_info()[2])
+                    origin="unpack_archive",
+                    message=f"Cannot untar {filename} ({e})",
+                ) from e
 
-        else:
+        elif ext == "zip":
             try:
                 with closing(E3ZipFile(filename, mode="r")) as zip_fd:
                     zip_fd.extractall(
@@ -246,8 +297,11 @@ def unpack_archive(
                     )
             except zipfile.BadZipfile as e:
                 raise ArchiveError(
-                    origin="unpack_archive", message=f"Cannot unzip {filename} ({e})",
-                ).with_traceback(sys.exc_info()[2])
+                    origin="unpack_archive",
+                    message=f"Cannot unzip {filename} ({e})",
+                ) from e
+        else:
+            assert_never()
 
         if remove_root_dir:
             # First check that we have only one dir in our temp destination,
@@ -311,11 +365,10 @@ def create_archive(
 ) -> None:
     """Create an archive file (.tgz, .tar.gz, .tar or .zip).
 
-    On Windows, if the python tarfile and zipfile modules are available, the
-    python implementation is used to create the archive.  On others system,
-    create_archive spawn tar, gzip or zip as it is twice faster that the python
-    implementation. If the tar, gzip or zip binary is not found, the python
-    implementation is used.
+    The Python implementations (tarfile and zipfile) are used on all platforms.
+    Spawning tar, gzip or zip on Linux could be faster, however it has the
+    disadvantage of not using the same implementation across platforms, hence
+    the choice to only use the Python implementations.
 
     :param filename: archive to create
     :param from_dir: directory to pack (full path)
@@ -356,7 +409,9 @@ def create_archive(
             tar_format = "w:gz"
         elif ext == "tar.bz2":
             tar_format = "w:bz2"
-        else:  # defensive code
-            raise ArchiveError(f"unsupported format {tar_format}")
+        elif ext == "tar.xz":
+            tar_format = "w:xz"
+        else:
+            assert_never()
         with closing(tarfile.open(filepath, tar_format)) as tar_archive:
             tar_archive.add(name=from_dir, arcname=from_dir_rename, recursive=True)
