@@ -3,7 +3,8 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Optional
+    from typing import Any, Optional
+    from collections.abc import Callable
 
 
 class EntryPoint:
@@ -15,7 +16,7 @@ class EntryPoint:
 
     def __init__(
         self,
-        db: Dict[str, EntryPoint],
+        db: dict[str, EntryPoint],
         fun: Callable[[], None],
         kind: str,
         name: Optional[str] = None,
@@ -38,7 +39,7 @@ class EntryPoint:
         self.kind = kind
         self.description = description
         self.executed = False
-        assert self.name not in db, "duplicate entry point %s" % self.name
+        assert self.name not in db, f"duplicate entry point {self.name}"
         db[self.name] = self
 
     def execute(self) -> None:
@@ -58,7 +59,7 @@ class Machine(EntryPoint):
 
     def __init__(
         self,
-        db: Dict[str, EntryPoint],
+        db: dict[str, EntryPoint],
         fun: Callable[[], None],
         kind: str,
         platform: str,
@@ -84,11 +85,11 @@ class Machine(EntryPoint):
 
 
 def entry_point(
-    db: Dict[str, EntryPoint],
+    db: dict[str, EntryPoint],
     cls: Callable[..., EntryPoint],
     kind: str,
     *args: Any,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Callable:
     """Entry point decorator.
 
@@ -107,7 +108,9 @@ def entry_point(
     :param kwargs: additional information to store with the entry point
     """
 
-    def entry_point_dec(f, ldb=db, lcls=cls, lkind=kind, largs=args, lkwargs=kwargs):
+    def entry_point_dec(  # type: ignore
+        f, ldb=db, lcls=cls, lkind=kind, largs=args, lkwargs=kwargs
+    ):
         lcls(ldb, f, lkind, *largs, **lkwargs)
         return f
 
