@@ -9,7 +9,6 @@ from __future__ import annotations
 import abc
 import os
 import pickle
-import sys
 from collections import namedtuple
 
 from typing import TYPE_CHECKING
@@ -20,7 +19,7 @@ from e3.platform import Platform
 
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Iterable, List, Optional, Union
+    from typing import Any, Iterable, Optional, TypeVar
     from argparse import Namespace
 
 logger = e3.log.getLogger("env")
@@ -306,7 +305,7 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
 
         :return: a namedtuple suitable for a call to set_env
         """
-        result: List[Optional[str]] = []
+        result: list[Optional[str]] = []
         if not self.build.is_default:
             result.append(",".join([self.build.platform, self.build.os.version]))
         else:
@@ -332,7 +331,7 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
             result.append(None)
         return EnvInfo(*result)
 
-    def cmd_triplet(self) -> List[str]:
+    def cmd_triplet(self) -> list[str]:
         """Return command line parameters corresponding to current env.
 
         :return: a list of command line parameters
@@ -340,13 +339,13 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
         build_str, host_str, target_str = self.str_triplet()
         result = []
         if build_str is not None:
-            result.append("--build=%s" % build_str)
+            result.append(f"--build={build_str}")
 
         if host_str is not None:
-            result.append("--host=%s" % host_str)
+            result.append(f"--host={host_str}")
 
         if target_str is not None:
-            result.append("--target=%s" % target_str)
+            result.append(f"--target={target_str}")
 
         return result
 
@@ -435,7 +434,7 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
         self.add_search_path(self.dll_path_var, path, append)
 
     @property
-    def discriminants(self) -> List[str]:
+    def discriminants(self) -> list[str]:
         """Compute discriminants.
 
         :return: the list of discriminants associated with the current context
@@ -467,9 +466,6 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
         if self.target.os.name.lower() == "windows":
             discs.append("NT")
 
-        if (not self.is_cross and not self.is_canadian) and self.build.is_virtual:
-            discs.append("virtual_machine")  # all: no cover
-
         return discs
 
     @property
@@ -542,6 +538,10 @@ class AbstractBaseEnv(metaclass=abc.ABCMeta):
             return e
 
 
+if TYPE_CHECKING:
+    BaseEnv_T = TypeVar("BaseEnv_T", bound="BaseEnv")
+
+
 class BaseEnv(AbstractBaseEnv):
     """BaseEnv."""
 
@@ -565,10 +565,10 @@ class BaseEnv(AbstractBaseEnv):
         :param target: target architecture. If None then it is set to target
         """
         # class variable that holds the current environment
-        self._instance: Dict[str, Any] = {}
+        self._instance: dict[str, Any] = {}
 
         # class variable that holds the stack of saved environments state
-        self._context: List[Any] = []
+        self._context: list[Any] = []
         super().__init__(build, host, target)
 
     def __setattr__(self, name: str, value: Any) -> None:
@@ -581,17 +581,17 @@ class BaseEnv(AbstractBaseEnv):
         try:
             return self._instance[name]
         except KeyError as e:
-            raise AttributeError(e).with_traceback(sys.exc_info()[2])
+            raise AttributeError(e) from e
 
     def _items(self) -> Iterable[Any]:
         return iter(self._instance.items())
 
     def copy(
-        self,
+        self: BaseEnv_T,
         build: Optional[str] = None,
         host: Optional[str] = None,
         target: Optional[str] = None,
-    ) -> BaseEnv:
+    ) -> BaseEnv_T:
         """Copy an env.
 
         :param build: like build set_env parameter
@@ -599,22 +599,24 @@ class BaseEnv(AbstractBaseEnv):
         :param target: like target set_env parameter
         :return: a deep copy of the current env
         """
-        result = BaseEnv()
+        result = self.__class__()
         for k, v in self._items():
             setattr(result, k, v)
         result.set_env(build, host, target)
         return result
 
     @classmethod
-    def from_env(cls, env: Optional[Union[Env, BaseEnv]] = None) -> BaseEnv:
+    def from_env(
+        cls: type[BaseEnv_T], env: Optional[Env | BaseEnv] = None
+    ) -> BaseEnv_T:
         """Return a new BaseEnv object from an env.
 
         :param env: env. If None copy the current Env
         """
         if env is None:
-            return BaseEnv(build=Env().build, host=Env().host, target=Env().target)
+            return cls(build=Env().build, host=Env().host, target=Env().target)
         else:
-            return BaseEnv(build=env.build, host=env.host, target=env.target)
+            return cls(build=env.build, host=env.host, target=env.target)
 
 
 class Env(AbstractBaseEnv):
@@ -623,14 +625,14 @@ class Env(AbstractBaseEnv):
     Env is a singleton holding the current environment and platform
     information. It is set by e3.main when the --build/--host/--target option
     are passed to the command line and can be then changed by calling
-    :meth:`set_build`, :meth:`set_host`, and :meth:`set_target`.
+    py:meth:`set_build`, py:meth:`set_host`, and py:meth:`set_target`.
     """
 
     # class variable that holds the current environment
-    _instance: Dict[str, Any] = {}
+    _instance: dict[str, Any] = {}
 
     # class variable that holds the stack of saved environments state
-    _context: List[Any] = []
+    _context: list[Any] = []
 
     def __init__(self) -> None:
         """Initialize or reuse an existing Env object (singleton).
@@ -656,7 +658,7 @@ class Env(AbstractBaseEnv):
         try:
             return self._instance[name]
         except KeyError as e:
-            raise AttributeError(e).with_traceback(sys.exc_info()[2])
+            raise AttributeError(e) from e
 
     def _items(self) -> Iterable[Any]:
         return iter(self._instance.items())
