@@ -3,7 +3,6 @@ from __future__ import annotations
 import abc
 import json
 import os
-import sys
 import time
 import uuid
 from typing import TYPE_CHECKING
@@ -19,7 +18,9 @@ from e3.fs import mkdir
 logger = e3.log.getLogger("event")
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Optional, Tuple, Union
+    from types import TracebackType
+    from typing import Any, Optional
+    from collections.abc import Callable
 
 
 def unique_id() -> str:
@@ -47,7 +48,7 @@ class Event:
       * end_time: time at which the event was closed
     """
 
-    def __init__(self, name: str, uid: Optional[str] = None, **kwargs: Dict[str, str]):
+    def __init__(self, name: str, uid: Optional[str] = None, **kwargs: dict[str, Any]):
         """Initialize an Event object.
 
         :param name: name of the event (e.g. e3_test)
@@ -69,7 +70,7 @@ class Event:
         self.end_time: Optional[float] = None
 
         for key, value in list(kwargs.items()):
-            self._data[key] = value  # type: ignore
+            self._data[key] = value
 
         self.set_formatter("begin_time", self.format_date)
         self.set_formatter("end_time", self.format_date)
@@ -77,7 +78,12 @@ class Event:
     def __enter__(self) -> Event:
         return self
 
-    def __exit__(self, _type, _value, _tb):
+    def __exit__(
+        self,
+        _type: Optional[type[BaseException]],
+        _val: Optional[BaseException],
+        _tb: Optional[TracebackType],
+    ) -> None:
         self.close()
 
     def set_formatter(self, key: str, fun: Callable[[str, Any], dict]) -> None:
@@ -87,28 +93,28 @@ class Event:
         :param fun: a function that takes the key and the associated value
             and return a dict
         """
-        self._formatters[key] = fun  # type: ignore
+        self._formatters[key] = fun
 
-    def __setattr__(self, name: str, value: str) -> None:
+    def __setattr__(self, name: str, value: Any) -> None:
         """Store all attributes in the self._data dict."""
         # Once the event is closed disallow attributes modifications
         if self._closed:
             raise EventError(f"event {self.name} ({self.uid}) closed")
-        self._data[name] = value  # type: ignore
+        self._data[name] = value
 
-    def __getattr__(self, name: str) -> str:
+    def __getattr__(self, name: str) -> Any:
         """Attributes are retrieved in the _data internal dict."""
         try:
-            return self._data[name]  # type: ignore
+            return self._data[name]
         except KeyError as e:
-            raise AttributeError(e).with_traceback(sys.exc_info()[2])
+            raise AttributeError(e) from e
 
-    def get_attachments(self) -> Dict[str, Tuple[str, str]]:
+    def get_attachments(self) -> dict[str, tuple[str, str]]:
         """Return the list of attachments.
 
         :return: a list of tuple (path, sha1(path))
         """
-        return self._attachments  # type: ignore
+        return self._attachments
 
     def attach_file(self, path: str, name: str = "log") -> None:
         """Attach log file to the event.
@@ -121,7 +127,7 @@ class Event:
         """
         if self._closed:
             raise EventError(f"event {self.name} ({self.uid}) closed")
-        self._attachments[name] = (path, e3.hash.sha1(path))  # type: ignore
+        self._attachments[name] = (path, e3.hash.sha1(path))
 
     def close(self) -> None:
         """Close the event. Once done it is not supposed to be modified.
@@ -133,7 +139,7 @@ class Event:
             self.end_time = time.time()
             object.__setattr__(self, "_closed", True)
 
-    def format_date(self, key: str, value: Union[float, str]) -> Dict[str, str]:
+    def format_date(self, key: str, value: float | str) -> dict[str, str]:
         """Format timestamp fields.
 
         :param key: the data key
@@ -145,7 +151,7 @@ class Event:
         else:
             return {key: value}
 
-    def as_dict(self) -> Dict[str, str]:
+    def as_dict(self) -> dict[str, str]:
         """Convert the event data into a dict that can be serialized as json.
 
         For each key, value of the _data dict by default add them into the
@@ -155,9 +161,9 @@ class Event:
         :return: a dict
         """
         result = {}
-        for key, value in list(self._data.items()):  # type: ignore
+        for key, value in list(self._data.items()):
             if key in self._formatters:
-                d = self._formatters[key](key, value)  # type: ignore
+                d = self._formatters[key](key, value)
                 result.update(d)
             else:
                 result[key] = value
@@ -219,7 +225,7 @@ class EventHandler(metaclass=abc.ABCMeta):
         """Decode a config string into a dict.
 
         :param config_str: the string containing configuration information
-        :return: a dict that can be used as **kwargs for the handler
+        :return: a dict that can be used as ``**kwargs`` for the handler
             initialization method
         """
         return {}
@@ -246,7 +252,7 @@ class EventManager:
 
     def __init__(self) -> None:
         """Initialize a manager."""
-        self.handlers: Dict[str, EventHandler] = {}
+        self.handlers: dict[str, EventHandler] = {}
 
     def send_event(self, event: Event) -> bool:
         """Send an event to using all registered handlers.
@@ -309,11 +315,9 @@ class EventManager:
         :param var_name: the name of the variable
         """
         handler_cfg_str = os.environ.get(var_name, "")
-        handler_cfg_dict: Dict[str, str] = dict(
+        handler_cfg_dict: dict[str, str] = dict(
             [
-                el.split("=", 1)  # type: ignore
-                if "=" in el
-                else (el, "")
+                el.split("=", 1) if "=" in el else (el, "")  # type: ignore
                 for el in handler_cfg_str.split("|")
             ]
         )
