@@ -13,7 +13,8 @@ from e3.job import Job
 
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, FrozenSet, List, Optional, Tuple, Type
+    from typing import Any, FrozenSet, Optional
+    from collections.abc import Callable
     from e3.collection.dag import DAG
 
     JobProviderCallback = Callable[
@@ -35,7 +36,7 @@ class Scheduler:
         self,
         job_provider: JobProviderCallback,
         collect: Optional[CollectCallback] = None,
-        queues: Optional[Dict[str, int]] = None,
+        queues: Optional[dict[str, int]] = None,
         tokens: int = 1,
         job_timeout: int = DEFAULT_JOB_MAX_DURATION,
     ):
@@ -50,11 +51,11 @@ class Scheduler:
         :param queues: describes the list of queues handled by the
             scheduler. The format is a dictionary for which which
             keys are queue names and value the max number of tokens
-            available at a given time. if None then a single queue
+            available at a given time. if empty then a single queue
             called "default" is created. Its size is then given by
             the tokens parameter.
         :param tokens: number of tokens for the default queue. Relevant
-            only when queues is None
+            only when queues is empty
         :param job_timeout: maximum execution time for a job. The default
             is 24h. If set to None timeout are disabled but it also make
             the scheduller non interruptable when waiting for a job to
@@ -67,7 +68,7 @@ class Scheduler:
         else:
             self.collect = collect
 
-        self.active_jobs: List[Job] = []
+        self.active_jobs: list[Job] = []
         self.queued_jobs = 0
         self.all_jobs_queued = False
         self.message_queue: Queue[Any] = Queue()
@@ -78,11 +79,11 @@ class Scheduler:
         self.max_active_jobs = 0
 
         # Initialize named queues
-        self.queues: Dict[str, List[Tuple[int, int, Job]]] = {}
+        self.queues: dict[str, list[tuple[int, int, Job]]] = {}
         self.tokens = {}
         self.n_tokens = 0
 
-        if queues is None:
+        if not queues:
             # If no queues are specificied create a default one
             # with no name.
             queues = {"default": tokens}
@@ -127,7 +128,7 @@ class Scheduler:
             return self.job_provider(uid, data, predecessors, notify_end)
 
     @classmethod
-    def simple_provider(cls, job_class: Type[Job]) -> JobProviderCallback:
+    def simple_provider(cls, job_class: type[Job]) -> JobProviderCallback:
         """Return a simple provider based on a given Job class.
 
         :param job_class: a subclass of Job
@@ -203,7 +204,7 @@ class Scheduler:
                 p.interrupt()
                 self.safe_collect(p)
                 p.on_finish(self)
-            raise KeyboardInterrupt
+            raise
         self.stop_time = datetime.now()
 
     def push(self, job: Job) -> None:
