@@ -14,14 +14,15 @@ import re
 import shutil
 import stat
 import sys
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, overload
 
 import e3
 import e3.error
 import e3.log
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, List, Optional, Tuple, Union
+    from typing import Any, Literal, Optional
+    from collections.abc import Callable
 
 
 class OSFSError(e3.error.E3Error):
@@ -41,9 +42,7 @@ def cd(path: str) -> None:
         os.chdir(path)
     except Exception as e:
         logger.error(e, exc_info=True)
-        raise OSFSError(
-            origin="cd", message="can't chdir to %s\n" % path
-        ).with_traceback(sys.exc_info()[2])
+        raise OSFSError(origin="cd", message=f"can't chdir to {path}\n") from e
 
 
 def chmod(mode: str, filename: str) -> int:
@@ -95,7 +94,7 @@ def chmod(mode: str, filename: str) -> int:
                 elif permlist.isdigit():
                     raise OSFSError(
                         origin="chmod",
-                        message="numeric mode not supported," " use os.chmod instead",
+                        message="numeric mode not supported, use os.chmod instead",
                     )
                 else:
                     action_mask = 0
@@ -136,23 +135,33 @@ def chmod(mode: str, filename: str) -> int:
     return current_mode
 
 
-def df(path: str, full: bool = False) -> Union[int, Tuple]:
+@overload
+def df(path: str) -> int:
+    ...
+
+
+@overload
+def df(path: str, full: Literal[True]) -> tuple:
+    ...
+
+
+def df(path: str, full: bool = False) -> int | tuple:
     """Disk space available on the filesystem containing the given path.
 
     :param path: a path
     :param full: if True return full disk information otherwise only
         space left.
 
-    :return: either space left in Mo or a :py:func:`collections.namedtuple`
+    :return: either space left in Mo or a py:meth:`collections.namedtuple`
         with ``total``, ``used`` and ``free`` attributes. Each attribute is
         an int representing Mo.
     """
-    _ntuple_diskusage = collections.namedtuple("usage", "total used free")
+    _ntuple_diskusage = collections.namedtuple("_ntuple_diskusage", "total used free")
     if sys.platform == "win32":  # unix: no cover
         import ctypes
 
-        path = ctypes.c_wchar_p(path)
-        GetDiskFreeSpaceEx = ctypes.WINFUNCTYPE(
+        c_path = ctypes.c_wchar_p(path)
+        GetDiskFreeSpaceEx: Callable = ctypes.WINFUNCTYPE(
             ctypes.c_int,
             ctypes.c_wchar_p,
             ctypes.POINTER(ctypes.c_uint64),
@@ -164,14 +173,14 @@ def df(path: str, full: bool = False) -> Union[int, Tuple]:
             ((1, "path"), (2, "freeuserspace"), (2, "totalspace"), (2, "freespace")),
         )
 
-        def GetDiskFreeSpaceEx_errcheck(result, func, args):
+        def GetDiskFreeSpaceEx_errcheck(result, func, args):  # type: ignore
             del func
             if not result:  # defensive code
                 raise ctypes.WinError()
             return (args[1].value, args[2].value, args[3].value)
 
-        GetDiskFreeSpaceEx.errcheck = GetDiskFreeSpaceEx_errcheck
-        _, total, free = GetDiskFreeSpaceEx(path)
+        GetDiskFreeSpaceEx.errcheck = GetDiskFreeSpaceEx_errcheck  # type: ignore
+        _, total, free = GetDiskFreeSpaceEx(c_path)
         used = total - free
     else:  # windows: no cover
         # f_frsize = fundamental filesystem block size
@@ -188,7 +197,7 @@ def df(path: str, full: bool = False) -> Union[int, Tuple]:
     return free // (1024 * 1024)
 
 
-def __safe_unlink_func() -> Tuple[Callable[[str], None], Callable[[str], None]]:
+def __safe_unlink_func() -> tuple[Callable[[str], None], Callable[[str], None]]:
     """Provide a safe unlink function on windows.
 
     Note that all this is done to ensure that rm is working fine on Windows 7
@@ -312,11 +321,14 @@ def which(prog: str, paths: Optional[str] = None, default: Any = "") -> Any:
     def is_exe(file_path: str) -> bool:
         return os.path.isfile(file_path) and os.access(file_path, os.X_OK)
 
-    def possible_names(file_path: str) -> List[str]:
+    def possible_names(file_path: str) -> list[str]:
         names = [file_path]
         if sys.platform == "win32":  # unix: no cover
             names.extend(
-                [file_path + ext for ext in os.environ.get("PATHEXT", "").split(";")]
+                [
+                    file_path + ext
+                    for ext in os.environ.get("PATHEXT", ".EXE").split(";")
+                ]
             )
         return names
 
