@@ -70,7 +70,8 @@ def subprocess_setup() -> None:
     """
     # Set sigpipe only when set_sigpipe is True
     # This should fix HC16-020 and could be activated by default
-    signal.signal(signal.SIGPIPE, signal.SIG_DFL)  # all: no cover
+    if sys.platform != "win32":
+        signal.signal(signal.SIGPIPE, signal.SIG_DFL)  # all: no cover
 
 
 def get_rlimit(platform: Optional[str] = None) -> str:
@@ -81,7 +82,7 @@ def get_rlimit(platform: Optional[str] = None) -> str:
 
     from pkg_resources import resource_filename
 
-    return resource_filename(__name__, os.path.join("data", "rlimit-%s" % platform))
+    return resource_filename(__name__, os.path.join("data", f"rlimit-{platform}"))
 
 
 def quote_arg(arg: str) -> str:
@@ -139,12 +140,12 @@ def quote_arg(arg: str) -> str:
             # Also, it seems to be nicer to print new-line characters
             # as '\n' rather than as a new-line...
             arg = arg.replace("\n", r"'\n'")
-            return "'%s'" % arg
+            return f"'{arg}'"
     # No quoting needed.  Return the argument as is.
     return arg
 
 
-def to_cmd_lines(cmds: AnyCmdLine) -> List[CmdLine]:
+def to_cmd_lines(cmds: AnyCmdLine) -> list[CmdLine]:
     if isinstance(cmds[0], str):
         # Turn the simple command into a special case of
         # the multiple-commands case.  This will allow us
@@ -185,8 +186,8 @@ def enable_commands_handler(filename: str, mode: str = "a") -> logging.Handler:
     class CmdFilter(logging.Filter):
         """Keep only e3.os.process.cmdline records."""
 
-        def filter(self, record: logging.LogRecord) -> int:
-            return 1 if record.name == "e3." + CMD_LOGGER_NAME else 0
+        def filter(self, record: logging.LogRecord) -> bool:
+            return True if record.name == "e3." + CMD_LOGGER_NAME else False
 
     # Here we don't attach the handler directly to the cmdline logger. Indeed
     # in class like e3.Main we do attach handlers to the root logger. In that
@@ -216,9 +217,11 @@ class Run:
         (a command line passed in a list, or a list of command lines passed as
         a list of list).
     :ivar status: The exit status. As the exit status is only meaningful after
-        the process has exited, its initial value is None.  When a problem
-        running the command is detected and a process does not get
-        created, its value gets set to the special value 127.
+        the process has exited, its initial value is None (see __init__'s "bg"
+        parameter for more information about this).
+
+        When a problem running the command is detected and a process does not
+        get created, its value gets set to the special value 127.
     :ivar raw_out: process standard output as bytes (if instanciated with
         output = PIPE). Use self.out to get a decoded string.
     :ivar raw_err: same as raw_out but for standard error.
@@ -229,9 +232,9 @@ class Run:
         self,
         cmds: AnyCmdLine,
         cwd: Optional[str] = None,
-        output: Union[STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = PIPE,
-        error: Union[STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = STDOUT,
-        input: Union[DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = None,
+        output: STDOUT_VALUE | DEVNULL_VALUE | PIPE_VALUE | str | IO | None = PIPE,
+        error: STDOUT_VALUE | DEVNULL_VALUE | PIPE_VALUE | str | IO | None = STDOUT,
+        input: DEVNULL_VALUE | PIPE_VALUE | str | IO | None = None,  # noqa: A002
         bg: bool = False,
         timeout: Optional[int] = None,
         env: Optional[dict] = None,
@@ -256,7 +259,17 @@ class Run:
             stderr data from the applications should be captured into the same
             file handle as for stdout.
         :param input: same as output
-        :param bg: if True then run in background
+        :param bg: if True, then run the program in the background.
+            Running the program in the background means that this method
+            will return immediately after having spawned the process
+            and therefore that our "status" attribute will be unset (None).
+            Users who want to access the process' exit status therefore
+            need to call this object's "wait" method first.
+
+            Conversely, when this parameter is set to False, this object's
+            initialization will take care of the "wait", thus ensuring that
+            the process has terminated and our "status" attriute gets set
+            by the time it returns.
         :param timeout: limit execution time (in seconds), None means
             unlimited
         :param env: dictionary for environment variables (e.g. os.environ)
@@ -290,7 +303,8 @@ class Run:
             """
             if not parse_shebang:
                 # nothing to do
-                return cmd_line
+                return [which(cmd_line[0], default=cmd_line[0])] + cmd_line[1:]
+
             prog = which(cmd_line[0], default=None)
             if prog is None:
                 # Not found. Do not modify the command line
@@ -342,7 +356,7 @@ class Run:
 
         self.status: Optional[int] = None
         self.raw_out = b""
-        self.raw_err = b""
+        self.raw_err: Optional[bytes] = b""
         self.cmds = []
 
         if env is not None:
@@ -405,12 +419,14 @@ class Run:
                 self.internal = Popen(self.cmds[0], **popen_args)
 
             else:
-                runs: List[Popen] = []
+                runs: list[subprocess.Popen] = []
                 for index, cmd in enumerate(self.cmds):
                     if index == 0:
-                        stdin = self.input_file.fd
+                        stdin: int | IO[Any] = self.input_file.fd
                     else:
-                        stdin = runs[index - 1].stdout
+                        previous_stdout = runs[index - 1].stdout
+                        assert previous_stdout is not None
+                        stdin = previous_stdout
 
                     # When connecting two processes using a Pipe don't use
                     # universal_newlines mode. Indeed commands transmitting
@@ -471,13 +487,15 @@ class Run:
         return bytes_as_str(self.raw_out)
 
     @property
-    def err(self) -> str:
+    def err(self) -> Optional[str]:
         """Process error as string.
 
         Attempt is done to decode as utf-8 the output. If the output is not in
         utf-8 a string representation will be returned
         (see e3.text.bytes_as_str).
         """
+        if self.raw_err is None:
+            return None
         return bytes_as_str(self.raw_err)
 
     def command_line_image(self) -> str:
@@ -494,7 +512,7 @@ class Run:
         self.error_file.close()
         self.input_file.close()
 
-    def __error(self, error: Exception, cmds: List[CmdLine]) -> None:
+    def __error(self, error: Exception, cmds: list[CmdLine]) -> None:
         """Set pid to -1 and status to 127 before closing files."""
         self.close_files()
         logger.error(error)
@@ -506,9 +524,7 @@ class Run:
             """
             logger.error("%s not found", path)
             e3.log.debug("PATH=%s", os.environ["PATH"])
-            raise OSError(
-                errno.ENOENT, "No such file or directory, %s not found" % path
-            )
+            raise OSError(errno.ENOENT, f"No such file or directory, {path} not found")
 
         # Try to send an helpful message if one of the executable has not
         # been found.
@@ -535,7 +551,7 @@ class Run:
         ):
             self.status = self.internal.wait()
         else:
-            tmp_input: Optional[Union[str, bytes]] = None
+            tmp_input: Optional[str | bytes] = None
             if self.input_file.fd == subprocess.PIPE:
                 tmp_input = self.input_file.get_command()
 
@@ -599,7 +615,7 @@ class Run:
         else:
             return self.internal.is_running()
 
-    def children(self) -> List[psutil.Process]:
+    def children(self) -> list[Any]:
         """Return list of child processes (using psutil)."""
         if psutil is None:  # defensive code
             raise NotImplementedError("Run.children() require psutil")
@@ -613,12 +629,12 @@ class File:
         """Create a new File.
 
         :param name: can be PIPE, STDOUT, a filename string, an opened fd, a
-            python file object, or a command to pipe (if starts with |)
+            python file object, or a command to pipe (if starts with ``|``)
         :param mode: can be 'r' or 'w' if name starts with + the mode will be
             a+
         """
         assert mode in "rw", "Mode should be r or w"
-        self.fd: Union[int, IO[str]]
+        self.fd: int | IO[str]
 
         self.name = name
         self.to_close = False
@@ -663,7 +679,7 @@ class WaitError(Exception):
     pass
 
 
-def wait_for_processes(process_list: List[Run], timeout: float) -> Optional[int]:
+def wait_for_processes(process_list: list[Run], timeout: float) -> Optional[int]:
     """Wait for several processes spawned with Run.
 
     :param process_list: a list of Run objects
@@ -676,22 +692,23 @@ def wait_for_processes(process_list: List[Run], timeout: float) -> Optional[int]
         return None
 
     start = time.time()
-    remain = timeout
 
     if sys.platform == "win32":  # unix: no cover
         from e3.os.windows.process import process_exit_code, wait_for_objects
 
+        remain = int(timeout)
+
         handles = [int(p.internal._handle) for p in process_list]
 
         while True:
             try:
                 idx = wait_for_objects(handles, remain, False)
                 if idx is None:
-                    return
+                    return None
 
                 if process_exit_code(handles[idx]) is None:
                     # Process is still active so wait after updating timeout
-                    remain = timeout - time.time() + start
+                    remain = int(timeout - time.time() + start)
 
                     if remain <= 0:
                         # No remaining time
@@ -700,12 +717,14 @@ def wait_for_processes(process_list: List[Run], timeout: float) -> Optional[int]
                     # Process is exiting so finalize it by calling wait
                     process_list[idx].wait()
                     return idx
-            except OSError:
-                raise WaitError
+            except OSError as err:
+                raise WaitError from err
 
     else:  # windows: no cover
         import select
 
+        remain = timeout
+
         # Each time a SIGCHLD signal is received write into pipe. Use
         # then select which support timeout arguments to wait.
         fd_r, fd_w = os.pipe()
@@ -750,7 +769,7 @@ def wait_for_processes(process_list: List[Run], timeout: float) -> Optional[int]
             signal.signal(signal.SIGCHLD, 0)
             os.close(fd_r)
             os.close(fd_w)
-    return None
+        return None
 
 
 def is_running(pid: int) -> bool:
@@ -762,6 +781,8 @@ def is_running(pid: int) -> bool:
         from e3.os.windows.native_api import Access, NT
         from e3.os.windows.process import process_exit_code
 
+        if TYPE_CHECKING:
+            assert NT.OpenProcess is not None
         handle = NT.OpenProcess(Access.PROCESS_QUERY_INFORMATION, False, pid)
 
         try:
@@ -770,6 +791,8 @@ def is_running(pid: int) -> bool:
             return process_exit_code(handle) is None
 
         finally:
+            if TYPE_CHECKING:
+                assert NT.Close is not None
             NT.Close(handle)
 
     else:  # windows: no cover
@@ -782,7 +805,7 @@ def is_running(pid: int) -> bool:
         return True
 
 
-def kill_process_tree(pid: Union[int, psutil.Process], timeout: int = 3) -> bool:
+def kill_process_tree(pid: int | Any, timeout: int = 3) -> bool:
     """Kill a hierarchy of processes.
 
     :param pid: pid of the toplevel process
@@ -817,7 +840,7 @@ def kill_process_tree(pid: Union[int, psutil.Process], timeout: int = 3) -> bool
 
     def on_terminate(p: str) -> None:
         """Log info when a process terminate."""
-        logger.info("process %s killed", p)
+        logger.debug("process %s killed", p)
 
     try:
         gone, alive = psutil.wait_procs(
