@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 import os
 import struct
 from ctypes import (
@@ -10,7 +12,6 @@ from ctypes import (
     sizeof,
 )
 from ctypes.wintypes import HANDLE
-from typing import Any, Callable, Optional
 
 import e3.log
 from e3.os.windows.native_api import (
@@ -27,6 +28,11 @@ from e3.os.windows.native_api import (
     UnicodeString,
 )
 
+if TYPE_CHECKING:
+    from typing import Any, Optional
+    from collections.abc import Callable
+
+
 logger = e3.log.getLogger("os.windows.fs")
 
 
@@ -41,8 +47,8 @@ class WithOpenFile:
         self.shared_access = shared_access
         self.open_options = open_options
 
-    def __call__(self, f):
-        def wrapper(obj, *args, **kwargs):
+    def __call__(self, f: Callable) -> Any:
+        def wrapper(obj, *args, **kwargs):  # type: ignore
             should_open = not obj.handle
             if should_open:
                 obj.open(self.desired_access, self.shared_access, self.open_options)
@@ -74,10 +80,10 @@ class NTFile:
         :param filename: path to the file or basename if parent is not None
         :param parent: the parent NTFile object
         """
-        self.handle = None  # type: Optional[HANDLE]
+        self.handle: Optional[HANDLE] = None
         if parent is None:
             self.path = os.path.abspath(filename)
-            self.nt_filename = UnicodeString("\\??\\%s" % self.path)
+            self.nt_filename = UnicodeString(f"\\??\\{self.path}")
             self.parent_handle = None
         else:
             assert "\\" not in filename and "/" not in filename
@@ -99,7 +105,10 @@ class NTFile:
         # the user has sufficient rights. It has no effect otherwise
         self.open_options = OpenOptions.BACKUP_INTENT
 
-    def __str__(self):
+        # Used by is_dir_empty to store the latest seen file in a given directory
+        self.is_dir_empty_last_seen_file: Optional[str] = None
+
+    def __str__(self) -> str:
         result = [
             "{:<20}: {}".format("path", self.path),
             "{:<20}: {}".format("nt_filename", self.nt_filename),
@@ -143,18 +152,18 @@ class NTFile:
         if status < 0:
             raise NTException(
                 status=status,
-                message="cannot open file %s" % self.path,
+                message=f"cannot open file {self.path}",
                 origin="NTFile.open",
             )
 
-    def close(self):
+    def close(self) -> None:
         """Close file.
 
         If a call has been done to open, close the handle associated
         with the file and reset the handle to None.
         """
         if self.handle:
-            NT.Close(self.handle)
+            NT.Close(self.handle)  # type: ignore
             self.handle = None
 
     @property
@@ -171,7 +180,7 @@ class NTFile:
         if not status:
             raise NTException(
                 status=status,
-                message="cannot find volume for %s" % self.path,
+                message=f"cannot find volume for {self.path}",
                 origin="NTFile.volume_path",
             )
         return result.value
@@ -206,7 +215,7 @@ class NTFile:
 
         return result.index_number
 
-    def read_attributes_internal(self):
+    def read_attributes_internal(self) -> None:
         """Retrieve file basic attributes (internal function).
 
         The function is used internally to check file basic attributes
@@ -214,16 +223,17 @@ class NTFile:
         attributes are stored in basic_info Python attribute. It requires
         less rights than the read_attributes method.
         """
-        status = NT.QueryAttributesFile(pointer(self.attr), pointer(self.basic_info))
+        query_attr_file: Callable = NT.QueryAttributesFile  # type: ignore
+        status = query_attr_file(pointer(self.attr), pointer(self.basic_info))
         if status < 0:
             raise NTException(
                 status=status,
-                message="cannot query attributes %s" % self.path,
+                message=f"cannot query attributes {self.path}",
                 origin="NTFile.read_attributes_internal",
             )
 
     @WithOpenFile(Access.READ_ATTRS)
-    def read_attributes(self):
+    def read_attributes(self) -> FileInfo.Basic:
         """Retrieve file basic information.
 
         It updates the basic_info attribute including timestamp information
@@ -231,7 +241,7 @@ class NTFile:
         :raise: NTException
         """
         result = FileInfo.Basic()
-        status = NT.QueryInformationFile(
+        status = NT.QueryInformationFile(  # type: ignore
             self.handle,
             pointer(self.io_status),
             pointer(result),
@@ -250,12 +260,12 @@ class NTFile:
         return result
 
     @WithOpenFile(Access.WRITE_ATTRS | Access.READ_ATTRS)
-    def write_attributes(self):
+    def write_attributes(self) -> None:
         """Update file attributes.
 
         :raise: NTException
         """
-        set_infofile: Callable = NT.SetInformationFile
+        set_infofile: Callable = NT.SetInformationFile  # type: ignore
         status = set_infofile(
             self.handle,
             pointer(self.io_status),
@@ -266,7 +276,7 @@ class NTFile:
         if status < 0:
             raise NTException(
                 status=status,
-                message="cannot write attributes to %s" % self.path,
+                message=f"cannot write attributes to {self.path}",
                 origin="NTFile.write_attributes",
             )
         self.read_attributes()
@@ -312,9 +322,9 @@ class NTFile:
         :param replace: if True replace the target file if it exists
         :raise: NTException
         """
-        file_target = "\\??\\%s" % os.path.abspath(filename)
+        file_target = f"\\??\\{os.path.abspath(filename)}"
         target = file_target.encode("utf_16_le")
-        s = "?PL%ss" % len(target)
+        s = f"?PL{len(target)}s"
         b = create_string_buffer(struct.calcsize(s))
         b.raw = struct.pack(s, replace, 0, len(target), target)
 
@@ -334,7 +344,7 @@ class NTFile:
                 origin="NTFile.rename",
             )
 
-    def move_to_trash(self):
+    def move_to_trash(self) -> None:
         """Move file to trash location.
 
         :raise: NTException
@@ -342,13 +352,13 @@ class NTFile:
         self.rename(self.trash_path, True)
 
     @WithOpenFile(Access.DELETE, Share.DELETE)
-    def dispose(self):
+    def dispose(self) -> None:
         """Remove the file (low level).
 
         The remove is effective on call to close method
         """
         fd = FileInfo.Disposition(1)
-        status = NT.SetInformationFile(
+        status = NT.SetInformationFile(  # type: ignore
             self.handle,
             pointer(self.io_status),
             pointer(fd),
@@ -402,7 +412,7 @@ class NTFile:
         if status < 0:
             raise NTException(
                 status=status,
-                message="can't read dir %s" % self.path,
+                message=f"can't read dir {self.path}",
                 origin="NTFile.iterate_on_dir",
             )
 
@@ -444,21 +454,30 @@ class NTFile:
     def is_dir_empty(self) -> bool:
         """Check if dir is empty.
 
-        :return: True if the directory is empty
+        :return: True if the directory is empty. If not empty return False and set
+            is_dir_empty_last_seen_file attribute to the first file found in the
+            directory.
         :raise: NTException
         """
+        self.is_dir_empty_last_seen_file = None
 
-        def check_file(filename, parent):
+        def check_file(filename: str, parent: Optional[NTFile]) -> tuple[bool, bool]:
+            """Check if file is pending deletion.
+
+            :return: if file is pending deletion return True and continue
+                iteration, otherwise return False and interrupt iterate_on_dir.
+            """
             f = NTFile(filename, parent)
             try:
                 f.read_attributes_internal()
             except NTException:
                 return True, False
+            self.is_dir_empty_last_seen_file = filename
             return False, True
 
         return self.iterate_on_dir(check_file, True)
 
-    def unlink(self):
+    def unlink(self) -> None:
         """Remove file safely.
 
         :raise: NTException
@@ -510,13 +529,13 @@ class NTFile:
                     raise
 
             # Wait a few ms before attempting again to open the file
-            NT.Sleep(5)
+            NT.Sleep(5)  # type: ignore
             try_counter -= 1
 
         if try_counter == 0:
             raise NTException(
                 status=1,
-                message="cannot open file %s for deletion" % self.path,
+                message=f"cannot open file {self.path} for deletion",
                 origin="NTFile.unlink",
             )
 
@@ -529,7 +548,8 @@ class NTFile:
                 if self.is_dir and not self.is_dir_empty:
                     raise NTException(
                         status=1,
-                        message="directory not empty: %s" % self.path,
+                        message=f"directory not empty: {self.path} "
+                        f"(file '{self.is_dir_empty_last_seen_file}' found)",
                         origin="NTFile.unlink",
                     )
 
@@ -557,9 +577,10 @@ class NTFile:
                         if not self.is_dir_empty:
                             raise NTException(
                                 status=e.status,
-                                message="dir %s is not empty" % self.path,
+                                message=f"dir {self.path} is not empty "
+                                f"(file '{self.is_dir_empty_last_seen_file}' found)",
                                 origin="NTFile.unlink",
-                            )
+                            ) from e
                     elif e.status == Status.CANNOT_DELETE:  # defensive code
                         # At this stage we are sure that the file is not
                         # read_only but it seems that we can get this error
@@ -578,7 +599,7 @@ class NTFile:
                             break
                         raise
 
-                NT.Sleep(5)
+                NT.Sleep(5)  # type: ignore
                 try_counter -= 1
 
         finally:
