@@ -3,6 +3,7 @@ from __future__ import annotations
 import ast
 import os
 import re
+import struct
 import sys
 from enum import Enum
 from dataclasses import asdict
@@ -12,7 +13,7 @@ from typing import TYPE_CHECKING
 import e3.log
 
 if TYPE_CHECKING:
-    from typing import List, Optional
+    from typing import Optional
 
 logger = e3.log.getLogger("e3.sys")
 
@@ -95,7 +96,7 @@ class RewriteImportRule:
                         pass
                     elif self.action == self.RuleAction.reject:
                         raise RewriteNodeError(
-                            "Rejected import found in ast: %s" % ast.dump(node)
+                            f"Rejected import found in ast: {ast.dump(node)}"
                         )
                     else:  # defensive code
                         raise ValueError("unknown action %s", self.action)
@@ -111,7 +112,7 @@ class RewriteImportNodeTransformer(ast.NodeTransformer):
     Currently only the RewriteImportRule are supported.
     """
 
-    def __init__(self, rules: List[RewriteImportRule]):
+    def __init__(self, rules: list[RewriteImportRule]):
         """Load a set of rules.
 
         :param rules: list of rule objects
@@ -244,7 +245,12 @@ def interpreter(prefix: Optional[str] = None) -> str:
         if os.path.exists(python3):
             return python3
         else:
-            return os.path.join(prefix, "python.exe")
+            # Might be the python location when in a venv
+            python3 = os.path.join(prefix, "Scripts", "python.exe")
+            if os.path.exists(python3):
+                return python3
+            else:
+                return os.path.join(prefix, "python.exe")
     else:  # windows: no cover
         python3 = os.path.join(prefix, "bin", "python3")
         if os.path.exists(python3):
@@ -253,7 +259,7 @@ def interpreter(prefix: Optional[str] = None) -> str:
             return os.path.join(prefix, "bin", "python")
 
 
-def python_script(name: str, prefix: Optional[str] = None) -> List[str]:
+def python_script(name: str, prefix: Optional[str] = None) -> list[str]:
     """Return path to scripts contained in this Python distribution.
 
     :param name: the script name
@@ -292,7 +298,7 @@ def python_script(name: str, prefix: Optional[str] = None) -> List[str]:
         # 3- a .exe without a side python script
         script = (
             os.path.join(prefix, name)
-            if os.path.basename(prefix) == "scripts"
+            if os.path.basename(prefix).lower() == "scripts"
             else os.path.join(prefix, "Scripts", name)
         )
 
@@ -318,3 +324,121 @@ def python_script(name: str, prefix: Optional[str] = None) -> List[str]:
             return [interpreter(prefix), script]
     else:
         return [interpreter(prefix), os.path.join(prefix, "bin", name)]
+
+
+def is_console() -> bool:
+    """Check if stdin is a console or not.
+
+    :return: True if stdin is an interactive console
+    """
+    stdin_fd = sys.stdin.fileno()
+
+    # First check using isatty call. On Windows isatty will return True only
+    # if in a Win32 console (for example cygwin or msys ptys are not win32
+    # consoles).
+    is_tty = os.isatty(stdin_fd)  # type: ignore
+    if is_tty:
+        return True
+
+    if sys.platform == "win32":  # unix: no cover
+
+        from msvcrt import get_osfhandle
+        from e3.os.windows.object import object_name
+
+        stdin_name = object_name(get_osfhandle(stdin_fd))  # type: ignore
+        if re.match(r"\\Device\\NamedPipe\\(cygwin|msys).*-pty.*$", stdin_name):
+            return True
+        else:
+            return False
+    else:  # win32: no cover
+        return False
+
+
+def relocate_python_distrib(
+    python_distrib_dir: Optional[str] = None,
+    freeze: bool = False,
+    platform: Optional[str] = None,
+) -> None:
+    """Adjust shebangs to freeze a Python distrib or make it relocatable.
+
+    :param python_distrib_dir: Python distrib prefix. If None the current
+        Python is used.
+    :param freeze: if True use absolute paths in shebangs. If not use relative paths.
+    :param platform: if None use sys.platform. Otherwise should be a platform value
+        as returned by sys.platform.
+    """
+    if platform is None:
+        platform = sys.platform
+
+    if python_distrib_dir is None:
+        python_distrib_dir = sys.prefix
+
+    if platform == "win32":  # unix: no cover
+        script_dir = os.path.join(python_distrib_dir, "Scripts")
+    else:  # win32: no cover
+        script_dir = os.path.join(python_distrib_dir, "bin")
+
+    # The python interpreter of the selected distribution
+    python_binary = interpreter(prefix=python_distrib_dir)
+
+    # Search all scripts
+    for fname in os.listdir(script_dir):
+        script_path = os.path.join(script_dir, fname)
+
+        if not os.path.isfile(script_path):
+            continue
+
+        with open(script_path, "rb") as fd:
+            content = fd.read()
+
+        # By default shebang is found in file first line
+        shebang_offset = 0
+
+        if not content.startswith(b"#!"):
+            if platform != "win32":  # win32: no cover
+                continue
+            else:  # unix: no cover
+                # Check if the file is windows executable with an embedded script.
+
+                # Windows executable starts with a MS-DOS header. Location
+                # of PE header can be found at offset 0x3c.
+                pe_offset = struct.unpack_from("I", content, offset=0x3C)[0]
+                if content[pe_offset : pe_offset + 4] != b"PE\0\0":
+                    # Not a PE file
+                    continue
+
+                # Does the executable contains a shebang ?
+                offsets = [m.start() for m in re.finditer(b"#!.*python", content)]
+
+                if not offsets:
+                    # No python shebang
+                    continue
+
+                assert len(offsets) == 1
+
+                shebang_offset = offsets[0]
+
+        launcher = content[:shebang_offset]
+        payload = content[shebang_offset:]
+
+        script_offset = payload.find(b"\n")
+        shebang = payload[:script_offset]
+        payload = payload[script_offset:]
+
+        if b"python" not in shebang:
+            continue
+
+        # At this stage we are sure that this is a Python script
+        if freeze:
+            shebang = b"#!" + python_binary.encode("utf-8")
+        else:
+            shebang = os.path.basename(python_binary).encode("utf-8")
+            if platform == "win32":  # unix: no cover
+                shebang = b"#!" + shebang
+            else:  # win32: no cover
+                shebang = b"#!/usr/bin/env " + shebang
+
+        with open(script_path, "wb") as fd:
+            fd.write(launcher)
+            fd.write(shebang)
+            fd.write(payload)
