@@ -1,7 +1,6 @@
+from __future__ import annotations
 import os
 
-from typing import Dict
-
 import e3.electrolyt.plan as plan
 from e3.anod.context import AnodContext, SchedulingError
 from e3.anod.error import AnodError
@@ -21,7 +20,7 @@ class TestContext:
         :param reject_duplicates: whether to reject duplicates in plan
         """
 
-        def repo_conf(name: str) -> Dict[str, str]:
+        def repo_conf(name: str) -> dict[str, str]:
             return {"vcs": "git", "url": name, "branch": "master"}
 
         # Create a context for a x86-linux machine
@@ -80,10 +79,10 @@ class TestContext:
         assert len(result) == 5, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec1.source.spec1-src",
-            "mylinux.x86-linux.spec1.source.sources",
+            "x86-linux.spec1.source.spec1-src",
+            "x86-linux.spec1.source.sources",
             "checkout.spec1-git",
-            "mylinux.x86-linux.spec1.upload_src.spec1-src",
+            "x86-linux.spec1.upload_src.spec1-src",
         }
 
     def test_add_anod_action2(self):
@@ -96,9 +95,9 @@ class TestContext:
         assert len(result) == 5, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec2.build",
+            "x86-linux.spec2.build",
             "source_get.spec2-src",
-            "mylinux.x86-linux.spec2.source_install.spec2-src",
+            "x86-linux.spec2.source_install.spec2-src",
             "download.spec2-src",
         }
 
@@ -144,9 +143,9 @@ class TestContext:
         assert len(result) == 4, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec3.build",
-            "mylinux.x86-linux.spec3.install",
-            "mylinux.x86-linux.spec3.upload_bin",
+            "x86-linux.spec3.build",
+            "x86-linux.spec3.install",
+            "x86-linux.spec3.upload_bin",
         }
 
     def test_add_anod_action4(self):
@@ -160,9 +159,9 @@ class TestContext:
         assert len(result) == 4, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec4.build",
-            "mylinux.x86-linux.spec4.install",
-            "mylinux.x86-linux.spec4.upload_bin",
+            "x86-linux.spec4.build",
+            "x86-linux.spec4.install",
+            "x86-linux.spec4.upload_bin",
         }
 
     def test_add_anod_action4_2(self):
@@ -174,8 +173,8 @@ class TestContext:
         assert len(result) == 3, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec4.download_bin",
-            "mylinux.x86-linux.spec4.install",
+            "x86-linux.spec4.download_bin",
+            "x86-linux.spec4.install",
         }
 
     def test_add_anod_action4_3(self):
@@ -187,7 +186,7 @@ class TestContext:
         assert len(result) == 2, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec4.test",
+            "x86-linux.spec4.test",
         }
 
     def test_add_anod_action5(self):
@@ -200,8 +199,8 @@ class TestContext:
         assert len(result) == 3, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec5.build",
-            "mylinux.x86-linux.spec5.upload_bin",
+            "x86-linux.spec5.build",
+            "x86-linux.spec5.upload_bin",
         }
 
     def test_add_anod_action6(self):
@@ -214,7 +213,7 @@ class TestContext:
         assert len(result) == 2, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec6.build",
+            "x86-linux.spec6.build",
         }
 
     def test_add_anod_action6_2(self):
@@ -233,7 +232,7 @@ class TestContext:
         assert len(result) == 2, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec6.build",
+            "x86-linux.spec6.build",
         }
 
     def test_add_anod_action7(self):
@@ -244,7 +243,7 @@ class TestContext:
         assert len(result) == 2, result.as_dot()
         assert set(result.vertex_data.keys()) == {
             "root",
-            "mylinux.x86-linux.spec7.build",
+            "x86-linux.spec7.build",
         }
 
     def test_add_anod_action8(self):
@@ -286,7 +285,7 @@ class TestContext:
             ac.schedule(ac.always_download_source_resolver)
 
         assert "has a build_tree dependency on spec3" in str(err)
-        assert 'anod_build("spec3", qualifier="foo"' ', build="x86-linux")' in str(err)
+        assert 'anod_build("spec3", qualifier="foo", build="x86-linux")' in str(err)
 
         ac.add_anod_action(
             "spec3",
@@ -364,8 +363,25 @@ class TestContext:
         result = ac.schedule(ac.always_download_source_resolver)
         keys = set(result.vertex_data.keys())
         assert len(keys) == 3, keys
-        assert "mylinux.x86-linux.spec13.download_bin" in keys
-        assert "mylinux.x86-linux.spec13.install" in keys
+        assert "x86-linux.spec13.download_bin" in keys
+        assert "x86-linux.spec13.install" in keys
+
+    def test_source_fails_when_missing_source_primitive(self):
+        """Source action should fail when the source primitive is undefined.
+
+        Check that a SchedulingError is thrown if the source primitive is absent and a
+        source packaging action is required.
+        """
+        ac = self.create_context()
+
+        # `anod source` should fail on a spec without a source primitive
+        with pytest.raises(
+            SchedulingError,
+            match=r"spec missing_source_primitive does not support primitive source",
+        ):
+            ac.add_anod_action(
+                "missing_source_primitive", env=ac.default_env, primitive="source"
+            )
 
     def test_add_anod_action_unmanaged_source(self):
         """Check no source creation for thirdparties."""
@@ -376,7 +392,7 @@ class TestContext:
         result = ac.schedule(ac.always_download_source_resolver)
         keys = set(result.vertex_data.keys())
         assert len(keys) == 2, keys
-        assert "mylinux.x86-linux.spec-unmanaged-source.source.wheel.whl" not in keys
+        assert "x86-linux.spec-unmanaged-source.source.wheel.whl" not in keys
 
     def test_add_anod_action_managed_source(self):
         """Check no source creation for thirdparties."""
@@ -388,7 +404,7 @@ class TestContext:
         keys = set(result.vertex_data.keys())
         assert len(keys) == 5, keys
         assert "checkout.a-git" in keys
-        assert "mylinux.x86-linux.spec-managed-source.source.a-src" in keys
+        assert "x86-linux.spec-managed-source.source.a-src" in keys
 
     def test_dag_2_plan(self):
         """Check that we can extract values from plan in final dag.
@@ -429,12 +445,19 @@ class TestContext:
         for a in ("anod_build", "anod_install", "anod_source", "anod_test"):
             cm.register_action(a, anod_action)
 
+        # Also register unsupported actions to verify that this does not
+        # crash the plan parser
+        cm.register_action("anod_foo", anod_action)
+        cm.register_action("foo", lambda: None)
+
         # Create a simple plan
         content = [
             "def myserver():",
             '    anod_build("spec12", weathers="foo")',
             '    anod_build("spec10", weathers="foo")',
             '    anod_build("spec11", weathers="bar")',
+            '    anod_foo("spec666")',
+            "    foo()",
         ]
         with open("plan.txt", "w") as f:
             f.write("\n".join(content))
@@ -443,15 +466,7 @@ class TestContext:
 
         # Execute the plan and create anod actions
         for action in cm.execute(myplan, "myserver"):
-            primitive = action.action.replace("anod_", "", 1)
-            ac.add_anod_action(
-                name=action.module,
-                env=current_env if action.default_build else action,
-                primitive=primitive,
-                qualifier=action.qualifier,
-                plan_line=action.plan_line,
-                plan_args=action.plan_args,
-            )
+            ac.add_plan_action(action)
 
         # Create a reverse tag to have a working get_context
         # when looking for parameters such as weathers we want to
@@ -566,15 +581,7 @@ class TestContext:
 
         # Execute the plan and create anod actions
         for action in cm.execute(myplan, "myserver"):
-            primitive = action.action.replace("anod_", "", 1)
-            ac.add_anod_action(
-                name=action.module,
-                env=current_env if action.default_build else action,
-                primitive=primitive,
-                qualifier=action.qualifier,
-                plan_line=action.plan_line,
-                plan_args=action.plan_args,
-            )
+            ac.add_plan_action(action)
 
         for uid, _ in ac.tree:
             if uid.endswith("sources"):
@@ -634,15 +641,7 @@ class TestContext:
             # Execute the plan and create anod actions
             # Execute the plan and create anod actions
             for action in cm.execute(myplan, "myserver"):
-                primitive = action.action.replace("anod_", "", 1)
-                ac.add_anod_action(
-                    name=action.module,
-                    env=current_env if action.default_build else action,
-                    primitive=primitive,
-                    qualifier=action.qualifier,
-                    plan_line=action.plan_line,
-                    plan_args=action.plan_args,
-                )
+                ac.add_plan_action(action)
 
             for uid, _ in ac.tree:
                 if uid.endswith("build"):
@@ -651,15 +650,7 @@ class TestContext:
 
             with pytest.raises(SchedulingError):
                 for action in cm.execute(myplan, "myserver"):
-                    primitive = action.action.replace("anod_", "", 1)
-                    ac.add_anod_action(
-                        name=action.module,
-                        env=current_env if action.default_build else action,
-                        primitive=primitive,
-                        qualifier=action.qualifier,
-                        plan_line=action.plan_line,
-                        plan_args=action.plan_args,
-                    )
+                    ac.add_plan_action(action)
 
     def test_plan_call_args(self):
         """Retrieve call args values."""
@@ -692,3 +683,12 @@ class TestContext:
             "The spec duplicate_dep has two dependencies with the same "
             "local_name attribute (spec3)" in str(err)
         )
+
+    def test_add_anod_action_missing_src_pkg_dep(self, caplog):
+        ac = self.create_context()
+        ac.add_anod_action("missing_src_pkg_dep", env=ac.default_env, primitive="build")
+        assert (
+            "source a-src coming from missing_src_pkg_dep_src"
+            " but there is no source_pkg dependency for"
+            " missing_src_pkg_dep_src in build_deps" in caplog.text
+        )
