@@ -5,7 +5,7 @@ import threading
 import time
 
 import requests_toolbelt.multipart
-from e3.net.http import HTTPSession
+from e3.net.http import HTTPSession, HTTPError
 
 try:
     from http.server import BaseHTTPRequestHandler, HTTPServer
@@ -112,7 +112,7 @@ class TestHTTP:
         with HTTPSession():
             pass
 
-    def test_retry(self):
+    def test_retry(self, socket_enabled):
         def func(server, base_url):
             with HTTPSession() as session:
                 session.set_max_retries(base_url, connect=5)
@@ -124,7 +124,7 @@ class TestHTTP:
 
         run_server(RetryHandler, func)
 
-    def test_content_dispo(self):
+    def test_content_dispo(self, socket_enabled):
         def func(server, base_url):
             with HTTPSession() as session:
                 result = session.download_file(base_url + "dummy", dest=".")
@@ -135,7 +135,7 @@ class TestHTTP:
 
         run_server(ContentDispoHandler, func)
 
-    def test_content_validation(self):
+    def test_content_validation(self, socket_enabled):
         def validate(path):
             return False
 
@@ -148,15 +148,24 @@ class TestHTTP:
 
         run_server(ContentDispoHandler, func)
 
-    def test_error(self):
+    def test_error(self, socket_enabled):
         def func(server, base_url):
             with HTTPSession() as session:
+                # first test with no exception on error
                 result = session.download_file(base_url + "dummy", dest=".")
                 assert result is None
+                # second test with error as exception
+                try:
+                    result = session.download_file(
+                        base_url + "dummy", dest=".", exception_on_error=True
+                    )
+                    raise AssertionError("exception not raised")
+                except HTTPError as e:
+                    assert e.status == 500
 
         run_server(ServerErrorHandler, func)
 
-    def test_fallback(self):
+    def test_fallback(self, socket_enabled):
         def func(server, base_url):
             def inner_func(server2, base_url2):
                 logging.info(f"servers: {base_url}, {base_url2}")
@@ -171,7 +180,7 @@ class TestHTTP:
 
         run_server(ServerErrorHandler, func)
 
-    def test_content_abort(self):
+    def test_content_abort(self, socket_enabled):
         def func(server, base_url):
             def inner_func(server2, base_url2):
                 logging.info(f"servers: {base_url}, {base_url2}")
@@ -187,7 +196,7 @@ class TestHTTP:
 
         run_server(ServerErrorHandler, func)
 
-    def test_post_stream_data(self):
+    def test_post_stream_data(self, socket_enabled):
         def outter_func(nok_server, nok_url):
             def func(server, url):
                 with HTTPSession(base_urls=[nok_url, url]) as session:
